unit codecraft.explorerbar.items;

interface
uses
   Windows,Classes,Controls, Graphics,ActnList,SysUtils,Types,JPEG,
   codecraft.explorerbar.types, codecraft.explorerbar;

type
   // text classes ---------------------------------------------------------------
   TGroupMenuItemViewer = class(TCustomBarItemViewer)
   public
      procedure Draw(Bitmap: TBitmap; var X, Y, Width: Integer); override;
      procedure Measure(Bitmap: TBitmap; var X, Y, Width: Integer); override;
   end;

   TGroupMenuItem = class(TExplorerBarGroupItem)
   private
      fBold: Boolean;
   public
      constructor Create(); override;
      destructor Destroy; override;
      function GetItemViewerClass(Sender:TObject):TCustomBarItemViewer; override;
      function OnItemClick(Sender: TObject): Integer; override;
      property Bold:Boolean read fBold write fBold;

   end;

   TTextLineItemViewer = class(TCustomBarItemViewer)
   public
      procedure Draw(Bitmap: TBitmap; var X: Integer; var Y: Integer; var Width: Integer); override;
      procedure Measure(Bitmap: TBitmap; var X: Integer; var Y: Integer; var Width: Integer); override;
   end;

   TGroupTextLineItem = class(TExplorerBarGroupItem)
   private
      fFontSize: Integer;
      fTextAlignment: TAlignment;
   public
      constructor Create(); override;
      function GetItemViewerClass(Sender: TObject): TCustomBarItemViewer; override;
      property FontSize:Integer read fFontSize write fFontSize;
      property TextAlignment:TAlignment read fTextAlignment write fTextAlignment;
   end;

   TCategoryItemViewer = class(TCustomBarItemViewer)
   public
      procedure Draw(Bitmap: TBitmap; var X: Integer; var Y: Integer; var Width: Integer); override;
      procedure Measure(Bitmap: TBitmap; var X: Integer; var Y: Integer; var Width: Integer); override;
   end;

   TGroupCategoryItem = class(TExplorerBarGroupItem)
   private
      fUnderlined: Boolean;
      fPaddingAfter: Integer;
      fPaddingBefore: Integer;
   public
      constructor Create(); override;
      function GetItemViewerClass(Sender: TObject): TCustomBarItemViewer; override;
      property Underlined:Boolean read fUnderlined write fUnderlined;
      property PaddingBefore:Integer read fPaddingBefore write fPaddingBefore;
      property PaddingAfter:Integer read fPaddingAfter write fPaddingAfter;
   end;
   // text classes ---------------------------------------------------------------

   // button classes --------------------------------------
   TCheckBoxItemViewer = class(TCustomBarItemViewer)
   private
      procedure DrawCheck(Canvas: TCanvas; R: TRect; AState: TCheckBoxState; AEnabled: Boolean);
   public
      procedure Draw(Bitmap: TBitmap; var X, Y, Width: Integer); override;
      procedure Measure(Bitmap: TBitmap; var X, Y, Width: Integer); override;
   end;

   TGroupCheckBoxItem = class(TExplorerBarGroupItem)
   private
      FChecked: Boolean;
      FEnabled: boolean;
      procedure SetEnabled(Value: boolean);
      procedure SetChecked(Value: boolean);
   public
      constructor Create(); override;
      function GetItemViewerClass(Sender:TObject):TCustomBarItemViewer; override;
      function OnItemClick(Sender:TObject):Integer; override;
      property Checked: boolean read FChecked write SetChecked;
      property Enabled: boolean read FEnabled write SetEnabled;
   end;

   {Radio Buttons --------------------------------------------------------------}
   TGroupOptionButtonsItemViewer = class(TCustomBarItemViewer)
   private
      procedure DrawRadio(Canvas: TCanvas; R: TRect; AState: TCheckBoxState; AEnabled: Boolean);
   public
      procedure Draw(Bitmap: TBitmap; var X, Y, Width: Integer); override;
      procedure Measure(Bitmap: TBitmap; var X, Y, Width: Integer); override;
   end;

   TGroupOptionButtonsItem = class(TExplorerBarGroupItem)
   private
      FEnabled: boolean;
      fItemIndex: Integer;
      fItems: TStringList;
      procedure SetEnabled(Value: boolean);
      procedure set_ItemIndex(const Value: Integer);
   public
      constructor Create(); override;
      destructor Destroy; override;
      function GetItemViewerClass(Sender:TObject):TCustomBarItemViewer; override;
      function OnItemClick(Sender:TObject):Integer; override;
      procedure DoMouseDown(Sender:TObject;X,Y:Integer); override;
      property Items:TStringList read fItems write fItems;
      property ItemIndex:Integer read fItemIndex write set_ItemIndex;
   end;
   {Radio Buttons --------------------------------------------------------------}

   TButtonItemViewer = class(TCustomBarItemViewer)
   private
      procedure DrawButton(Canvas: TCanvas; R: TRect; AEnabled,aPushed,aHot: Boolean);
   public
      procedure Draw(Bitmap: TBitmap; var X: Integer; var Y: Integer; var Width: Integer); override;
      procedure Measure(Bitmap: TBitmap; var X: Integer; var Y: Integer; var Width: Integer); override;
   end;

   TGroupButtonItem = class(TExplorerBarGroupItem)
   public
      constructor Create(); override;
      function OnItemClick(Sender:TObject):Integer; override;
      function GetItemViewerClass(Sender:TObject):TCustomBarItemViewer; override;
   end;
   // button classes --------------------------------------
  // graphic classes ---------------------------------------------------
   TGraphicItemViewer = class(TCustomBarItemViewer)
   private
   protected
   public
      procedure Draw(Bitmap: TBitmap; var X: Integer; var Y: Integer; var Width: Integer); override;
      procedure Measure(Bitmap: TBitmap; var X: Integer; var Y: Integer; var Width: Integer); override;
   end;

   TCustomGroupGraphicItem = class(TExplorerBarGroupItem)
   private
      fStretch: Boolean;
      fGraphic: TGraphic;
      procedure SetStretch(const Value: Boolean);
   public
      constructor Create(); override;
      destructor Destroy; override;
      procedure LoadFromFile(const lpzFilename:string); virtual;
      procedure SetGraphic(Value: TGraphic); virtual;
      property Graphic:TGraphic read fGraphic write fGraphic;
      function GetItemViewerClass(Sender: TObject): TCustomBarItemViewer; override;
      property Stretch:Boolean read fStretch write SetStretch;
   end;

   TGroupBitmapItem = class(TCustomGroupGraphicItem)
   public
      constructor Create(); override;
      procedure SetGraphic(Value: TGraphic); override;
      function OnItemClick(Sender: TObject): Integer; override;
   end;

   TGroupJPEGItem = class(TCustomGroupGraphicItem)
   public
      constructor Create(); override;
      procedure SetGraphic(Value: TGraphic); override;
   end;
  // graphic classes ---------------------------------------------------

    // spacer classes ---------------------------------------------------
   TSeparatorItemViewer = class(TCustomBarItemViewer)
   public
      procedure Draw(Bitmap: TBitmap; var X: Integer; var Y: Integer; var Width: Integer); override;
      procedure Measure(Bitmap: TBitmap; var X: Integer; var Y: Integer; var Width: Integer); override;
   end;

   TGroupSeparatorItem = class(TExplorerBarGroupItem)
   private
      FColor: TColor;
   public
      constructor Create(); override;
      function GetItemViewerClass(Sender: TObject): TCustomBarItemViewer; override;
      property Color: TColor read FColor write fColor;
   end;

   TSpacerItemViewer = class(TCustomBarItemViewer)
   public
      procedure Draw(Bitmap: TBitmap; var X: Integer; var Y: Integer; var Width: Integer); override;
      procedure Measure(Bitmap: TBitmap; var X: Integer; var Y: Integer; var Width: Integer); override;
   end;

   TGroupSpacerItem = class(TExplorerBarGroupItem)
   public
      constructor Create(); override;
      function GetItemViewerClass(Sender: TObject): TCustomBarItemViewer; override;
   end;
    // spacer classes ---------------------------------------------------

    // control classes ---------------------------------------------------
   TProgressBarItemViewer = class(TCustomBarItemViewer)
   public
      procedure Draw(Bitmap: TBitmap; var X: Integer; var Y: Integer; var Width: Integer); override;
      procedure Measure(Bitmap: TBitmap; var X: Integer; var Y: Integer; var Width: Integer); override;
   end;

   TSimpleProgressBarItemViewer = class(TCustomBarItemViewer)
   public
      procedure Draw(Bitmap: TBitmap; var X: Integer; var Y: Integer; var Width: Integer); override;
      procedure Measure(Bitmap: TBitmap; var X: Integer; var Y: Integer; var Width: Integer); override;
   end;

   TCustomDrawProgressBarItemViewer = class(TCustomBarItemViewer)
   public
      procedure Draw(Bitmap: TBitmap; var X: Integer; var Y: Integer; var Width: Integer); override;
      procedure Measure(Bitmap: TBitmap; var X: Integer; var Y: Integer; var Width: Integer); override;
   end;

  {a base progress bar}
   TCustomGroupProgressBarItem = class(TExplorerBarGroupItem)
   private
      FMin, FMax, FPosition: Integer;
      FHeight: Integer;
      FBorderColor: TColor;
      FFillColor: TColor;
      procedure SetMin(Value: Integer);
      procedure SetMax(Value: Integer);
      procedure SetPosition(Value: Integer);
   public
      constructor Create(); override;
      property Min: Integer read FMin write SetMin;
      property Max: Integer read FMax write SetMax;
      property Position: Integer read FPosition write SetPosition;
      property BorderColour:TColor read FBorderColor write fBorderColor;
      property FillColour:TColor read FFillColor write FFillColor;
   end;

  {a progress bar}
   TGroupProgressBarItem = class(TCustomGroupProgressBarItem)
   public
      function GetItemViewerClass(Sender: TObject): TCustomBarItemViewer; override;
   end;
  {a simple progress bar}
   TGroupSimpleProgressBarItem = class(TCustomGroupProgressBarItem)
   public
      constructor Create(); override;
      function GetItemViewerClass(Sender: TObject): TCustomBarItemViewer; override;
   end;

  {a custom draw progress bar}
   TGroupCustomDrawProgressBarItem = class(TCustomGroupProgressBarItem)
   public
      function GetItemViewerClass(Sender: TObject): TCustomBarItemViewer; override;
   end;

   {custom draw classes}
   TCustomDrawBarItemViewer = class(TCustomBarItemViewer)
   public
      procedure Draw(Bitmap: TBitmap; var X: Integer; var Y: Integer; var Width: Integer); override;
      procedure Measure(Bitmap: TBitmap; var X: Integer; var Y: Integer; var Width: Integer); override;
   end;

  {a custom draw  bar}
   TGroupCustomDrawBarItem = class(TCustomGroupProgressBarItem)
   public
      function GetItemViewerClass(Sender: TObject): TCustomBarItemViewer; override;
   end;
    // control classes ---------------------------------------------------

    // Date and Calendars ------------------------------------------------
   TMonthCalendarItemViewer = class(TCustomBarItemViewer)
   private
      fTheYear: Integer;
      fTheMonth: Integer;
      procedure PaintDay(Canvas:TCanvas;Day: Integer; DayRect: TRect);
      procedure PaintMonth(Canvas:TCanvas;MonthRect: TRect);
      function GetMonthRect(Bitmap: TBitmap; var X: Integer; var Y: Integer; var Width: Integer):TRect;
      function GetMonthName(Month:Integer):String;
      function GetDayInWeekChar(DayInWeek: Integer):Char;
      procedure SetPastMonth(var AYear, AMonth: Integer; Decrease: Boolean = True);
   public
      procedure Draw(Bitmap: TBitmap; var X: Integer; var Y: Integer; var Width: Integer); override;
      procedure Measure(Bitmap: TBitmap; var X: Integer; var Y: Integer; var Width: Integer); override;
      property TheMonth:Integer read fTheMonth write fTheMonth;
      property TheYear:Integer read fTheYear write fTheYear;
   end;

   TGroupMonthCalendarItem = class(TExplorerBarGroupItem)
   private
      FCalendarDate: TDateTime;
      procedure SetCalendarDate(const Value: TDateTime);
   public
      constructor Create(); override;
      destructor Destroy; override;
      function OnItemClick(Sender:TObject):Integer; override;
      function GetItemViewerClass(Sender: TObject): TCustomBarItemViewer; override;
      property CalendarDate:TDateTime read FCalendarDate write SetCalendarDate;
   end;
    // Date and Calendars ------------------------------------------------

    // action classes ---------------------------------------------------
   TGroupActionItem = class;
   TGroupItemActionLink = class(TActionLink)
   private
      fActionItem:TGroupActionItem;
   public
      procedure AssignClient(AClient: TObject); override;
      procedure SetEnabled(Value: Boolean); override;
      function Update:Boolean; override;
      procedure Change; override;
   end;

   TActionItemViewer = class(TCustomBarItemViewer)
   public
      procedure Draw(Bitmap: TBitmap; var X: Integer; var Y: Integer; var Width: Integer); override;
      procedure Measure(Bitmap: TBitmap; var X: Integer; var Y: Integer; var Width: Integer); override;
   end;

   TGroupActionItem = class(TExplorerBarGroupItem)
   private
      fActionLink:TGroupItemActionLink;
      fActionImages:TImageList;
      procedure SetAction(const Value: TAction);
      function GetAction: TAction;
   public
      constructor Create(); override;
      destructor Destroy; override;
      function OnItemClick(Sender:TObject):Integer; override;
      function GetItemViewerClass(Sender: TObject): TCustomBarItemViewer; override;
      property Action:TAction read GetAction write SetAction;
   end;
    // action classes ---------------------------------------------------

   TExplorerItemFactory = class(TObject)
   private
   protected
   public
      class function ProgressBar(aGroup:TExplorerBarGroup;dwMax,dwMin,dwPosition:Integer):TGroupProgressBarItem;
      class function SimpleProgressBar(aGroup:TExplorerBarGroup;dwMax,dwMin,dwPosition:Integer):TGroupSimpleProgressBarItem;
      class function CustomProgressBar(aGroup:TExplorerBarGroup):TGroupCustomDrawProgressBarItem;
      class function CustomBarItem(aGroup:TExplorerBarGroup):TGroupCustomDrawBarItem;
      class function Menu(aGroup:TExplorerBarGroup;const lpzCaption:string; dwIdentifier,dwIconIndex:Integer; const Wrap:Boolean = false):TGroupMenuItem;
      class function Bitmap(aGroup:TExplorerBarGroup;aBitmap:TBitmap; HotTrack,Stretch:Boolean; dwIdentifier:Integer):TGroupBitmapItem;
      class function Text(aGroup:TExplorerBarGroup;const lpzCaption:string; aFontColour:TColor;aFontStyle:TFontStyles; Wrap:Boolean;dwIconIndex:Integer; const dwFontSize:Integer = 8; Aligned:TAlignment = taLeftJustify):TGroupTextLineItem;
      class function Spacer(aGroup:TExplorerBarGroup; dwHeight:Integer = -1):TGroupSpacerItem;
      class function Separator(aGroup:TExplorerBarGroup; dwColour:TColor = -1):TGroupSeparatorItem;
      class function Action(aGroup:TExplorerBarGroup;AAction: TAction; AImageList: TImageList):TGroupActionItem; overload;
      class function Action(aGroup:TExplorerBarGroup;AAction: TAction; dwIconIndex:Integer):TGroupActionItem; overload;
      class function Category(aGroup:TExplorerBarGroup;const lpzCaption:string; aFontColour:TColor;aFontStyle:TFontStyles; Underline:Boolean):TGroupCategoryItem;
      class function Checkbox(aGroup:TExplorerBarGroup;const lpzCaption:string; aChecked:Boolean = True;aEnabled:Boolean = True; dwIdentifier:Integer = -1):TGroupCheckBoxItem;
      class function Field(aGroup:TExplorerBarGroup;const lpzCaption,lpzFieldName:string;aDataset:TObject; aFontColour:TColor;aFontStyle:TFontStyles; Wrap:Boolean;dwIconIndex:Integer; const dwFontSize:Integer = 8; Aligned:TAlignment = taLeftJustify):TGroupTextLineItem;
      class function RadioButtons(aGroup:TExplorerBarGroup;const lpzItems:TStrings; dwIndex:Integer):TGroupOptionButtonsItem;
      class function Button(aGroup:TExplorerBarGroup;const lpzCaption:string; dwIdentifier:Integer):TGroupButtonItem;
      class function MonthCalendar(aGroup:TExplorerBarGroup;const dwCalendarDate:TDateTime):TGroupMonthCalendarItem;
   end;


implementation
uses
   DB,Math,Dateutils,Themes,codecraft.explorerbar.consts, codecraft.explorerbar.utils;

function TGroupMenuItem.GetItemViewerClass(Sender: TObject): TCustomBarItemViewer;
begin
   result:= TGroupMenuItemViewer.Create(Self);
end;

constructor TGroupMenuItem.Create();
begin
   inherited;
   Bold:= True;
   HotTracking:= True;
end;


destructor TGroupMenuItem.Destroy;
begin
   if ExplorerGroup.ExplorerBar.HotArea = Self then ExplorerGroup.ExplorerBar.HotArea := nil;
end;

{ TGroupMenuItemViewer }

procedure TGroupMenuItemViewer.Draw(Bitmap: TBitmap; var X, Y, Width: Integer);
var
   IconX,IconY:Integer; r:TRect;
begin
   if bisHot in Item.State then
   begin
      Bitmap.Canvas.Font.Style:= [fsUnderline];
      Bitmap.Canvas.Font.Color := Item.ExplorerGroup.ExplorerBar.ColourSet.TextHot;
   end
   else
   begin
      Bitmap.Canvas.Font.Style:= Item.FontStyle;
      Bitmap.Canvas.Font.Color := Item.ExplorerGroup.ExplorerBar.ColourSet.Text;
   end;
   IconX:= X; IconY:= Y;
   DrawIcon(Bitmap,IconX,IconY,Width);
   SetRect(R,IconX,Y,Width, Y + TExplorerBarText.HeightOf(Bitmap.Canvas,Item.Caption,Width));
   DrawCaption(Bitmap,X,Y,r,0);
   inherited;
end;

procedure TGroupMenuItemViewer.Measure(Bitmap: TBitmap; var X, Y, Width: Integer);
begin
   inherited Measure(Bitmap,X,Y,Width);
end;

constructor TGroupTextLineItem.Create;
begin
   inherited;
   FontSize:= 8;
end;

function TGroupTextLineItem.GetItemViewerClass(Sender: TObject): TCustomBarItemViewer;
begin
   result:= TTextLineItemViewer.Create(Self);
end;

{ TTextLineItemViewer }

procedure TTextLineItemViewer.Draw(Bitmap: TBitmap; var X, Y, Width: Integer);
var
   IconX,IconY:Integer; r:TRect; dwSize:Integer;
begin
   dwSize:= Bitmap.Canvas.Font.Size;
   Bitmap.Canvas.Font.Size:= TGroupTextLineItem(Item).FontSize;
   Bitmap.Canvas.Font.Style:= Item.FontStyle;
   Bitmap.Canvas.Font.Color := Self.Item.FontColor;

   IconX:= X; IconY:= Y;
   DrawIcon(Bitmap,IconX,IconY,Width);
   if Item.WordWrap then
      SetRect(R,IconX,Y,Width, Y + TExplorerBarText.HeightOf(Bitmap.Canvas,Item.Caption,Width))
   else
      SetRect(R,IconX,Y,Width, Y + Bitmap.Canvas.TextHeight('Yy'));
   if TGroupTextLineItem(Item).TextAlignment = taCenter then
      SetRect(R,IconX + ((Width - Bitmap.Canvas.TextWidth(Item.Caption)) div 2) - X_OFFSET,Y,Width, Y + Bitmap.Canvas.TextHeight('Yy'));
//      bitmap.Canvas.Brush.Color:= clRed;
//      Bitmap.Canvas.FrameRect(r);
   DrawCaption(Bitmap,X,Y,r,0);

   inherited;
   Bitmap.Canvas.Font.Size:= dwSize;
end;

procedure TTextLineItemViewer.Measure(Bitmap: TBitmap; var X, Y, Width: Integer);
var
   dwSize:Integer;
begin
   dwSize:= Bitmap.Canvas.Font.Size;
   Bitmap.Canvas.Font.Size:= TGroupTextLineItem(Item).FontSize;
   inherited Measure(Bitmap,X,Y,Width);
   Bitmap.Canvas.Font.Size:= dwSize;
//   Inc(Y,Y_OFFSET div 2);
end;

constructor TGroupCategoryItem.Create();
begin
   inherited;
   FontColor := clWindowText;
   FontStyle:= [fsbold];
   Underlined:= True;
   PaddingBefore:= 10;
   PaddingAfter:= 0;
end;


function TGroupCategoryItem.GetItemViewerClass(Sender: TObject): TCustomBarItemViewer;
begin
   result:= TCategoryItemViewer.Create(Self);
end;

{ TCategoryItemViewer }

procedure TCategoryItemViewer.Draw(Bitmap: TBitmap; var X, Y, Width: Integer);
var
   IconX,IconY:Integer; r:TRect;
begin
   Bitmap.Canvas.Font.Style:= Item.FontStyle;
   Bitmap.Canvas.Font.Color := Item.FontColor;
   IconX:= X; IconY:= Y;
   DrawIcon(Bitmap,IconX,IconY,Width);
   Inc(Y,TGroupCategoryItem(Item).PaddingBefore);
   SetRect(R,IconX,Y,Width, Y + TExplorerBarText.HeightOf(Bitmap.Canvas,Item.Caption,Width));
   DrawCaption(Bitmap,X,Y,r,0);

   if TGroupCategoryItem(Item).Underlined then
   begin
      Bitmap.Canvas.Pen.Color:= Item.FontColor;
      Bitmap.Canvas.MoveTo(X + X_OFFSET,Y - 2);
      Bitmap.Canvas.LineTo(Width - 5,Y - 2);
      Inc(Y, Y_OFFSET div 2);
   end;
   Inc(Y,TGroupCategoryItem(Item).PaddingAfter);
   inherited;
end;

procedure TCategoryItemViewer.Measure(Bitmap: TBitmap; var X, Y, Width: Integer);
begin
   Inc(Y,TGroupCategoryItem(Item).PaddingBefore);
   inherited Measure(Bitmap,X,Y,Width);
   if TGroupCategoryItem(Item).Underlined then
   begin
      Inc(Y, Y_OFFSET div 2);
   end;
   Inc(Y,TGroupCategoryItem(Item).PaddingAfter);
end;

constructor TGroupCheckBoxItem.Create();
begin
   inherited;
   FChecked:= false;
   FEnabled:= true;
   Height:= 23;
   HotTracking:= True;
end;

procedure TGroupCheckBoxItem.SetEnabled(Value: boolean);
begin
   FEnabled:=Value;
   NotifyPaint;
end;

procedure TGroupCheckBoxItem.SetChecked(Value: boolean);
begin
   FChecked:=Value;
   NotifyPaint;
end;


function TGroupCheckBoxItem.GetItemViewerClass(Sender: TObject): TCustomBarItemViewer;
begin
   result:= TCheckBoxItemViewer.Create(Self);
end;

function TGroupCheckBoxItem.OnItemClick(Sender: TObject): Integer;
begin
   if Enabled then
   begin
      Self.Checked:= not Self.Checked;
      ExplorerGroup.ExplorerBar.DoOnItemClickEvent(Self,Self.Index,ExplorerGroup.Index);
   end;
end;

{ TCheckBoxItemViewer }

procedure TCheckBoxItemViewer.Draw(Bitmap: TBitmap; var X, Y, Width: Integer);
var
   CheckTop: Integer; ALine, EndStr: string; ARect: TRect;
begin
   with Bitmap.Canvas do
   begin
      if bisHot in Item.State then
      begin
         Font.Style := [fsUnderline];
         Font.Color := Item.ExplorerGroup.ExplorerBar.ColourSet.TextHot;
      end
      else
      begin
         Font.Style := [];
         Font.Color := Item.ExplorerGroup.ExplorerBar.ColourSet.Text;
      end;

      if not TGroupCheckBoxItem(Item).Enabled then
      begin
         Font.Style:=Font.Style-[fsUnderLine];
         Font.Color:= Item.ExplorerGroup.ExplorerBar.ColourSet.Text;
      end;

      CheckTop := Y;

      if (TGroupCheckBoxItem(Item).Checked) and (TGroupCheckBoxItem(Item).Enabled) then
         Font.Style := Font.Style +[fsBold]
      else
         Font.Style := Font.Style - [fsBold];

      ARect:= Rect(X + X_OFFSET, Y, X + X_OFFSET + Item.ExplorerGroup.ExplorerBar.ThemeElements.CheckBoxWidth , Y + Item.Height);
      if TGroupCheckBoxItem(Item).Checked then
         DrawCheck(Bitmap.Canvas, ARect, cbChecked, TGroupCheckBoxItem(Item).Enabled)
      else
         DrawCheck(Bitmap.Canvas, ARect, cbUnchecked, TGroupCheckBoxItem(Item).Enabled);
      ARect:= Rect(X + (X_OFFSET * 3), Y, Item.ExplorerGroup.Width - (X_OFFSET), Y + Item.Height);

      EndStr := Item.Caption;
      Brush.Style := bsClear;
      {$IFDEF VER150}
      DrawText(Handle,PChar(EndStr),-1,aRect,DT_SINGLELINE or DT_VCENTER or DT_LEFT);
      {$ELSE}
      TextRect(aRect,EndStr,[tfSingleLine,tfEndEllipsis,tfVerticalCenter,tfLeft]);
      {$ENDIF}
      Item.ClientAreaRectangle:= Rect(X + X_OFFSET, Y, Item.ExplorerGroup.Width - (X_OFFSET) , Y + Item.Height);
      Inc(Y,TextHeight('A') + LINE_Y_OFFSET);
   end;
//   Inc(Y, Y_OFFSET div 2);
   inherited;
end;

procedure TCheckBoxItemViewer.DrawCheck(Canvas: TCanvas; R: TRect; AState: TCheckBoxState; AEnabled: Boolean);
var
   DrawRect:TRect;DrawState:Integer; element:TThemedElementDetails;
begin
   DrawRect.Left := R.Left + (R.Right - R.Left - Item.ExplorerGroup.ExplorerBar.ThemeElements.CheckBoxWidth) div 2;
   DrawRect.Top := R.Top + (R.Bottom - R.Top - Item.ExplorerGroup.ExplorerBar.ThemeElements.CheckBoxWidth) div 2;
   DrawRect.Right := DrawRect.Left + Item.Width;
   DrawRect.Bottom := DrawRect.Top + Item.Height;

   if ThemeServices.ThemesEnabled then
   begin
      case AState of
         cbChecked   :element:= ThemeServices.GetElementDetails(tbCheckBoxCheckedNormal);
         cbUnchecked :element:= ThemeServices.GetElementDetails(tbCheckBoxUncheckedNormal);
         else
            element:= ThemeServices.GetElementDetails(tbCheckBoxUncheckedNormal);
      end;
      if not AEnabled then
         element:= ThemeServices.GetElementDetails(tbCheckBoxUncheckedDisabled);
      ThemeServices.DrawElement(Canvas.Handle,element,R);
   end
   else
   begin
      case AState of
         cbChecked:
            DrawState := DFCS_BUTTONCHECK or DFCS_CHECKED;
         cbUnchecked:
            DrawState := DFCS_BUTTONCHECK;
         else
            DrawState := DFCS_BUTTON3STATE or DFCS_CHECKED;
      end;
      if not AEnabled then
         DrawState := DrawState or DFCS_INACTIVE;
      DrawFrameControl(Canvas.Handle, DrawRect, DFC_BUTTON, DrawState);
   end;
end;

procedure TCheckBoxItemViewer.Measure(Bitmap: TBitmap; var X, Y, Width: Integer);
begin
   inherited Measure(Bitmap,X,Y,Width);
end;

{ TButtonItemViewer }

procedure TButtonItemViewer.Draw(Bitmap: TBitmap; var X, Y, Width: Integer);
var
   ARect: TRect; sCaption:String;
begin
   with Bitmap.Canvas do
   begin
      if bisHot in Item.State then
      begin
         Font.Color := Item.ExplorerGroup.ExplorerBar.ColourSet.TextHot;
      end
      else
      begin
         Font.Color := Item.ExplorerGroup.ExplorerBar.ColourSet.Text;
      end;
      if not Item.Enabled then
      begin
         Font.Color:= Item.ExplorerGroup.ExplorerBar.ColourSet.Text;
      end;

      sCaption:= Item.Caption;
      ARect:= Rect(X + X_OFFSET, Y, Item.ExplorerGroup.Width - X_OFFSET, Y +Item.Height);
      DrawButton(Bitmap.Canvas,ARect,Item.Enabled,bisPushed in Item.State,bisHot in Item.State);
      Item.ClientAreaRectangle:= ARect;
      Brush.Style := bsClear;
 //     Bitmap.Canvas.TextRect(ARect,sCaption,[tfSingleLine,tfCenter,tfEndEllipsis,tfVerticalCenter]);
   end;
   Inc(Y,Item.Height);
   Inc(Y, Y_OFFSET div 2);
//   inherited;
end;

procedure TButtonItemViewer.DrawButton(Canvas: TCanvas; R: TRect; AEnabled,aPushed,aHot: Boolean);
var
   DrawRect:TRect; element:TThemedElementDetails; DrawState:Integer;
begin
   DrawRect.Left := R.Left;
   DrawRect.Top := R.Top;
   DrawRect.Right := R.Right;
   DrawRect.Bottom := R.Bottom;

   if ThemeServices.ThemesEnabled then
   begin
      if not aEnabled then
         element:= ThemeServices.GetElementDetails(tbPushButtonDisabled)
      else
         if aPushed then
            element:= ThemeServices.GetElementDetails(tbPushButtonPressed)
         else
            if aHot then
               element:= ThemeServices.GetElementDetails(tbPushButtonHot)
            else
               element:= ThemeServices.GetElementDetails(tbPushButtonNormal);
      ThemeServices.DrawElement(Canvas.Handle,element,DrawRect);
   end
   else
   begin
      if aPushed then
         DrawState:= DFCS_BUTTONPUSH OR DFCS_PUSHED
      else
         DrawState := DFCS_BUTTONPUSH;
      if not AEnabled then DrawState := DrawState or DFCS_INACTIVE;
      DrawFrameControl(Canvas.Handle, DrawRect, DFC_BUTTON, DrawState);
   end;

end;

procedure TButtonItemViewer.Measure(Bitmap: TBitmap; var X, Y, Width: Integer);
begin
   Inc(Y,Item.Height);
   Inc(Y, Y_OFFSET div 2);
//   inherited;
end;


{ TGroupButtonItem }

constructor TGroupButtonItem.Create();
begin
   inherited;
   Height:= 23;
   HotTracking:= True;
end;

function TGroupButtonItem.GetItemViewerClass(Sender: TObject): TCustomBarItemViewer;
begin
   result:= TButtonItemViewer.Create(Self);
end;


function TGroupButtonItem.OnItemClick(Sender: TObject): Integer;
begin
   if Enabled then
   begin
      Self.State:= [bisPushed,bisHot];
      ExplorerGroup.ExplorerBar.DoOnItemClickEvent(Self,Self.Index,ExplorerGroup.Index);
      Self.State:= [bisHot];
   end;
end;

{ TGraphicItemViewer }

procedure TGraphicItemViewer.Draw(Bitmap: TBitmap; var X, Y, Width: Integer);
var
   W: Integer; r:TRect;
begin
   if TCustomGroupGraphicItem(Item).Graphic <> nil then
      if Item.ExplorerGroup.ExplorerBar.Width < DEFAULT_WIDTH then
      begin
         W := Item.ExplorerGroup.ExplorerBar.Width - SCROLLBAR_WIDTH - 4 * X_OFFSET;
         if TCustomGroupGraphicItem(Item).Stretch then
         begin
            Bitmap.Canvas.StretchDraw(Rect(X + (Width - W) div 2, Y, X + Width - (Width - W) div 2, Y + TCustomGroupGraphicItem(Item).Graphic.Height * W div TCustomGroupGraphicItem(Item).Graphic.Width), TCustomGroupGraphicItem(Item).Graphic);
            Item.ClientAreaRectangle:= Rect(X + (Width - W) div 2, Y, X + Width - (Width - W) div 2, Y + TCustomGroupGraphicItem(Item).Graphic.Height * W div TCustomGroupGraphicItem(Item).Graphic.Width);
            Inc(Y, TCustomGroupGraphicItem(Item).Graphic.Height * W div TCustomGroupGraphicItem(Item).Graphic.Width + Y_OFFSET);
         end
         else
         begin
            Bitmap.Canvas.Draw(X + (Width - W) div 2, Y, TCustomGroupGraphicItem(Item).Graphic);
            Item.ClientAreaRectangle:= Rect(X + (Width - W) div 2,Y, X + (Width - W) div 2 + TCustomGroupGraphicItem(Item).Graphic.Width,Y + TCustomGroupGraphicItem(Item).Graphic.Height);
            Inc(Y, TCustomGroupGraphicItem(Item).Graphic.Height + Y_OFFSET);
         end;
      end
      else
      begin
         Bitmap.Canvas.Draw(X + (Width - TCustomGroupGraphicItem(Item).Graphic.Width) div 2, Y, TCustomGroupGraphicItem(Item).Graphic);
         Item.ClientAreaRectangle:= Rect(X + (Width - TCustomGroupGraphicItem(Item).Graphic.Width) div 2,Y, X + (Width - TCustomGroupGraphicItem(Item).Graphic.Width) div 2 + TCustomGroupGraphicItem(Item).Graphic.Width,Y + TCustomGroupGraphicItem(Item).Graphic.Height);
         Inc(Y, TCustomGroupGraphicItem(Item).Graphic.Height + Y_OFFSET);
      end;

   if (Item.MouseInControl) and (Item.HotTracking) then
   begin
      Bitmap.Canvas.Brush.Color:= Item.ExplorerGroup.ExplorerBar.ColourSet.TextHot;
      r:= Item.ClientAreaRectangle;
      InflateRect(R,2,2);
      Bitmap.Canvas.FrameRect(R);
      Bitmap.Canvas.Brush.Style:= bsClear;
   end;
//   inherited;
end;

procedure TGraphicItemViewer.Measure(Bitmap: TBitmap; var X, Y, Width: Integer);
var
   W: Integer;
begin
   if TCustomGroupGraphicItem(Item).Graphic <> nil then
      if Item.ExplorerGroup.ExplorerBar.Width < DEFAULT_WIDTH then
      begin
         if TCustomGroupGraphicItem(Item).Stretch then
         begin
            W := Item.ExplorerGroup.ExplorerBar.Width - SCROLLBAR_WIDTH - 4 * X_OFFSET;
            Inc(Y, TCustomGroupGraphicItem(Item).Graphic.Height * W div TCustomGroupGraphicItem(Item).Graphic.Width + Y_OFFSET);
         end
         else
         begin
            Inc(Y, TCustomGroupGraphicItem(Item).Graphic.Height + Y_OFFSET);
         end;
      end
      else
         Inc(Y, TCustomGroupGraphicItem(Item).Graphic.Height + Y_OFFSET);

//   Inc(Y, Y_OFFSET);
//   inherited;
end;

{ TCustomGroupGraphicItem }

constructor TCustomGroupGraphicItem.Create;
begin
   inherited;
   Stretch:= True;
end;

destructor TCustomGroupGraphicItem.Destroy;
begin
   Graphic.Free;
   inherited;
end;

function TCustomGroupGraphicItem.GetItemViewerClass(Sender: TObject): TCustomBarItemViewer;
begin
   result:= TGraphicItemViewer.Create(Self);
end;

procedure TCustomGroupGraphicItem.LoadFromFile(const lpzFilename: string);
begin
   if FileExists(lpzFilename) then Graphic.LoadFromFile(lpzFilename);
end;


procedure TCustomGroupGraphicItem.SetGraphic(Value: TGraphic);
begin
end;

procedure TCustomGroupGraphicItem.SetStretch(const Value: Boolean);
begin
   fStretch := Value;
end;

constructor TGroupBitmapItem.Create();
begin
   inherited;
   Graphic:= TBitmap.Create;
end;

function TGroupBitmapItem.OnItemClick(Sender: TObject): Integer;
begin
   if Enabled then
   begin
      ExplorerGroup.ExplorerBar.DoOnItemClickEvent(Self,Self.Index,ExplorerGroup.Index);
   end;
end;

procedure TGroupBitmapItem.SetGraphic(Value: TGraphic);
begin
   inherited;
   if Value <> nil then
   begin
      if Stretch then
      begin
         Graphic.Width := DEFAULT_WIDTH - SCROLLBAR_WIDTH - 4 * X_OFFSET;
         Graphic.Height := Value.Height * (DEFAULT_WIDTH - SCROLLBAR_WIDTH - 4 * X_OFFSET) div Value.Width;
         TBitmap(Graphic).Canvas.StretchDraw(TBitmap(Graphic).Canvas.ClipRect, Value);
      end
      else
         TBitmap(Graphic).Assign(Value);
   end;
   NotifyPaint;
end;

constructor TGroupJPEGItem.Create();
begin
   inherited;
   Graphic:= TJPEGImage.Create;
end;


procedure TGroupJPEGItem.SetGraphic(Value: TGraphic);
begin
   if Value <> nil then
   begin
      Graphic.Assign(Value);
//      if Stretch then
//      begin
//           Graphic.Assign(Value);
////         Graphic.Width := DEFAULT_WIDTH - SCROLLBAR_WIDTH - 4 * X_OFFSET;
////         Graphic.Height := Value.Height * (DEFAULT_WIDTH - SCROLLBAR_WIDTH - 4 * X_OFFSET) div Value.Width;
////         TBitmap(Graphic).Canvas.StretchDraw(TBitmap(Graphic).Canvas.ClipRect, Value);
//      end
//      else
////         TBitmap(Graphic).Assign(Value);
   end;
   NotifyPaint;
end;


constructor TGroupSeparatorItem.Create();
begin
   inherited;
   Color := clNavy;
end;


function TGroupSeparatorItem.GetItemViewerClass(Sender: TObject): TCustomBarItemViewer;
begin
   result:= TSeparatorItemViewer.Create(Self);
end;

{ TSeparatorItemViewer }

procedure TSeparatorItemViewer.Draw(Bitmap: TBitmap; var X, Y, Width: Integer);
begin
   with Bitmap.Canvas do
   begin
      Pen.Color:= TGroupSeparatorItem(Item).Color;
      MoveTo(X + 5,Y + (Y_OFFSET div 2));
      LineTo(Width - 5,Y + (Y_OFFSET div 2));
      Inc(Y, Y_OFFSET);
   end;
end;

procedure TSeparatorItemViewer.Measure(Bitmap: TBitmap; var X, Y, Width: Integer);
begin
   Inc(Y, Y_OFFSET);
end;

constructor TGroupSpacerItem.Create();
begin
   inherited;
   Height:= Y_OFFSET;
end;

function TGroupSpacerItem.GetItemViewerClass(Sender: TObject): TCustomBarItemViewer;
begin
   result:= TSpacerItemViewer.Create(Self);
end;

{ TSpacerItemViewer }

procedure TSpacerItemViewer.Draw(Bitmap: TBitmap; var X, Y, Width: Integer);
begin
   Inc(Y, Item.Height);
//   inherited;
end;

procedure TSpacerItemViewer.Measure(Bitmap: TBitmap; var X, Y, Width: Integer);
begin
   Inc(Y, Item.Height);
//   inherited Measure(Bitmap,X,Y,Width);
end;


function TGroupProgressBarItem.GetItemViewerClass(Sender: TObject): TCustomBarItemViewer;
begin
   result:= TProgressBarItemViewer.Create(Self);
end;


{ TProgressBarItemViewer }

procedure TProgressBarItemViewer.Draw(Bitmap: TBitmap; var X, Y, Width: Integer);
var
   element:TThemedElementDetails; r:TRect; aItem:TGroupProgressBarItem;
begin
   aItem:= TGroupProgressBarItem(Item);
   with Bitmap.Canvas do
   begin
      r:= Rect(X + X_OFFSET, Y, X + Width - X_OFFSET, Y + Item.Height);

      if ThemeServices.ThemesEnabled then
      begin
         element:= ThemeServices.GetElementDetails(tpBar);
         ThemeServices.DrawElement(Handle,element,r);
         if (aItem.Position > 0) and (aItem.Max > 0) then
            r:= Rect(X + X_OFFSET + 2, Y + 2, X + X_OFFSET + 2 + Round((Width - 2 * X_OFFSET - 4) * (aItem.Position - aItem.Min) / (aItem.Max - aItem.Min)), Y + Item.Height - 2);
         element:= ThemeServices.GetElementDetails(tpChunk);
         ThemeServices.DrawElement(Handle,element,r);
      end
      else
      begin
         Pen.Color := TGroupProgressBarItem(Item).BorderColour;
         Rectangle(X + X_OFFSET, Y, X + Width - X_OFFSET, Y + Item.Height);
         Brush.Color := aItem.FillColour;
         if (aItem.Position > 0) and (aItem.Max > 0) then
            FillRect(Rect(X + X_OFFSET + 2, Y + 2, X + X_OFFSET + 2 + Round((Width - 2 * X_OFFSET - 4) * (aItem.Position - aItem.Min) / (aItem.Max - aItem.Min)), Y + Item.Height - 2));
      end;
      Inc(Y, Item.Height + Y_OFFSET);
      Item.ClientAreaRectangle:= Rect(X + X_OFFSET, Y, X + Width - X_OFFSET, Y + Item.Height);
   end;
//   inherited;
end;

procedure TProgressBarItemViewer.Measure(Bitmap: TBitmap; var X, Y, Width: Integer);
begin
   Inc(Y, Item.Height + Y_OFFSET);
//   inherited;
end;

{ TExActionItemLink }
function TGroupItemActionLink.Update:Boolean;
begin
   result:= inherited Update;
   fActionItem.Caption:= TAction(Self.Action).Caption;
   fActionItem.Enabled:= TAction(Self.Action).Enabled;
end;

procedure TGroupItemActionLink.AssignClient(AClient: TObject);
begin
   inherited;
   fActionItem:= TGroupActionItem(AClient);
end;

procedure TGroupItemActionLink.Change;
begin
   inherited;
   fActionItem.Caption:= TAction(Self.Action).Caption;
   fActionItem.Enabled:= TAction(Self.Action).Enabled;
end;

procedure TGroupItemActionLink.SetEnabled(Value: Boolean);
begin
   inherited;
   fActionItem.Enabled:= Value;
   fActionItem.FontStyle:= [];
end;

{ TExActionItem }

constructor TGroupActionItem.Create;
begin
   inherited;
   HotTracking:= True;
end;

destructor TGroupActionItem.Destroy;
begin
   if fActionLink <> nil then fActionLink.Free;
   inherited;
end;

procedure TGroupActionItem.SetAction(const Value: TAction);
begin
   if fActionLink = nil then
      fActionLink:= TGroupItemActionLink.Create(Self)
   else
   begin
      fActionLink.Free;
      fActionLink:= TGroupItemActionLink.Create(Self);
   end;
   fActionLink.Action:= Value;
   Self.Enabled:= GetAction.Enabled;
   Self.Caption:= GetAction.Caption;
end;

function TGroupActionItem.GetAction: TAction;
begin
   result:= TAction(fActionLink.Action);
end;

function TGroupActionItem.GetItemViewerClass(Sender: TObject): TCustomBarItemViewer;
begin
   result:= TActionItemViewer.Create(Self);
end;

function TGroupActionItem.OnItemClick(Sender: TObject): Integer;
begin
   if Self.Action.Enabled then
   begin
      Self.Action.Execute;
      ExplorerGroup.ExplorerBar.DoOnItemClickEvent(Self,Self.Index,ExplorerGroup.Index);
   end;
end;

{ TActionItemViewer }

procedure TActionItemViewer.Draw(Bitmap: TBitmap; var X, Y, Width: Integer);
var
   IconX,IconY:Integer; r:TRect;
begin
   if bisHot in Item.State then
   begin
      Bitmap.Canvas.Font.Style:= [fsUnderline];
      Bitmap.Canvas.Font.Color := Item.ExplorerGroup.ExplorerBar.ColourSet.TextHot;
   end
   else
   begin
      Bitmap.Canvas.Font.Style:= Item.FontStyle;
      Bitmap.Canvas.Font.Color := Item.ExplorerGroup.ExplorerBar.ColourSet.Text;
   end;
   if not TGroupActionItem(Item).Action.Enabled then
   begin
      Bitmap.Canvas.Font.Color:= clGrayText;
   end;

   IconX:= X; IconY:= Y;
   DrawIcon(Bitmap,IconX,IconY,Width);
   SetRect(R,IconX,Y,Width, Y + TExplorerBarText.HeightOf(Bitmap.Canvas,Item.Caption,Width));
   DrawCaption(Bitmap,X,Y,r,0);
   inherited;
end;

procedure TActionItemViewer.Measure(Bitmap: TBitmap; var X, Y, Width: Integer);
begin
   if TGroupActionItem(Item).Action <> nil then TGroupActionItem(Item).Action.Update;
   inherited Measure(Bitmap,X,Y,Width);
end;

{ TExplorerItemFactory }

class function TExplorerItemFactory.Action(aGroup: TExplorerBarGroup; AAction: TAction;AImageList: TImageList): TGroupActionItem;
var
   Bitmap:TBitmap;
begin
   result:= TGroupActionItem(aGroup.Items.Add(TGroupActionItem.Create));
   if AImageList <> nil then
   begin
      Bitmap:= TBitmap.Create;
      try
         AImageList.GetBitmap(AAction.ImageIndex,Bitmap);
         Result.Icon.Width:= Bitmap.Width;
         Result.Icon.Height:= Bitmap.Height;
         Result.Icon.Canvas.BrushCopy(Rect(0,0,Result.Icon.Width,Result.Icon.Height),Bitmap,Rect(0,0,Bitmap.Width,Bitmap.Height),clWhite);
         Result.Icon.Assign(Bitmap);
      finally
         Bitmap.Free;
      end;
   end;
   Result.Action:= AAction;
   Result.WordWrap:= True;
   Result.Hint:= AAction.Hint;
end;

class function TExplorerItemFactory.Action(aGroup: TExplorerBarGroup; AAction: TAction; dwIconIndex: Integer): TGroupActionItem;
var
   Bitmap:TBitmap;
begin
   result:= TGroupActionItem(aGroup.Items.Add(TGroupActionItem.Create));
   Result.IconIndex:= dwIconIndex;
   Result.Action:= AAction;
   Result.Hint:= AAction.Hint;
   Result.WordWrap:= True;
end;

class function TExplorerItemFactory.Bitmap(aGroup: TExplorerBarGroup; aBitmap: TBitmap; HotTrack,Stretch: Boolean; dwIdentifier: Integer): TGroupBitmapItem;
begin
   result:= TGroupBitmapItem(aGroup.Items.Add(TGroupBitmapItem.Create));
   Result.HotTracking:= HotTrack;
   Result.Stretch:= Stretch;
   Result.SetGraphic(aBitmap);
end;

class function TExplorerItemFactory.Button(aGroup: TExplorerBarGroup; const lpzCaption: string; dwIdentifier: Integer): TGroupButtonItem;
begin
   result:= TGroupButtonItem(aGroup.Items.Add(TGroupButtonItem.Create));
   Result.Caption:= lpzCaption;
end;

class function TExplorerItemFactory.Category(aGroup: TExplorerBarGroup; const lpzCaption: string; aFontColour: TColor; aFontStyle: TFontStyles; Underline:Boolean): TGroupCategoryItem;
begin
   result:= TGroupCategoryItem(aGroup.Items.Add(TGroupCategoryItem.Create));
   Result.Caption:= lpzCaption;
   if aFontColour > 0 then Result.FontColor:= aFontColour;
   Result.FontStyle:= aFontStyle;
   Result.Underlined:= Underline;
end;

class function TExplorerItemFactory.Checkbox(aGroup: TExplorerBarGroup; const lpzCaption: string; aChecked:Boolean = True;aEnabled:Boolean = True;dwIdentifier:Integer = -1): TGroupCheckBoxItem;
begin
   result:= TGroupCheckBoxItem(aGroup.Items.Add(TGroupCheckBoxItem.Create));
   Result.Caption:= lpzCaption;
   Result.Checked:= aChecked;
   Result.Enabled:= aEnabled;
   Result.Identifier:= dwIdentifier;
end;

class function TExplorerItemFactory.CustomBarItem(aGroup: TExplorerBarGroup): TGroupCustomDrawBarItem;
begin
   result:= TGroupCustomDrawBarItem(aGroup.Items.Add(TGroupCustomDrawBarItem.Create));
end;

class function TExplorerItemFactory.CustomProgressBar(aGroup: TExplorerBarGroup): TGroupCustomDrawProgressBarItem;
begin
   result:= TGroupCustomDrawProgressBarItem(aGroup.Items.Add(TGroupCustomDrawProgressBarItem.Create));
end;

class function TExplorerItemFactory.Field(aGroup: TExplorerBarGroup; const lpzCaption, lpzFieldName: string; aDataset: TObject; aFontColour: TColor; aFontStyle: TFontStyles; Wrap: Boolean;dwIconIndex: Integer; const dwFontSize: Integer; Aligned: TAlignment): TGroupTextLineItem;
begin
   result:= TGroupTextLineItem(aGroup.Items.Add(TGroupTextLineItem.Create));
   if aDataset <> nil then
   begin
      Result.Caption:= lpzCaption + ' ' + TDataSet(aDataset).FieldByName(lpzFieldName).AsString;
   end;

   if aFontColour > 0 then Result.FontColor:= aFontColour;
   Result.FontStyle:= aFontStyle;
   Result.WordWrap:= Wrap;
   Result.IconIndex:= dwIconIndex;
   Result.FontSize:= dwFontSize;
   if Aligned <> taLeftJustify then Result.TextAlignment:= Aligned;
end;

class function TExplorerItemFactory.Menu(aGroup: TExplorerBarGroup; const lpzCaption: string; dwIdentifier, dwIconIndex: Integer;const Wrap:Boolean = false): TGroupMenuItem;
begin
   result:= TGroupMenuItem(aGroup.Items.Add(TGroupMenuItem.Create));
   Result.Caption:= lpzCaption;
   Result.Identifier:= dwIdentifier;
   Result.IconIndex:= dwIconIndex;
   Result.WordWrap:= Wrap;
end;

class function TExplorerItemFactory.MonthCalendar(aGroup: TExplorerBarGroup; const dwCalendarDate: TDateTime): TGroupMonthCalendarItem;
begin
   result:= TGroupMonthCalendarItem(aGroup.Items.Add(TGroupMonthCalendarItem.Create));
end;

class function TExplorerItemFactory.ProgressBar(aGroup: TExplorerBarGroup; dwMax, dwMin, dwPosition: Integer): TGroupProgressBarItem;
begin
   result:= TGroupProgressBarItem(aGroup.Items.Add(TGroupProgressBarItem.Create));
   Result.Max:= dwMax;
   Result.Min:= dwMin;
   Result.Position:= dwPosition;
end;

class function TExplorerItemFactory.RadioButtons(aGroup: TExplorerBarGroup; const lpzItems: TStrings; dwIndex: Integer): TGroupOptionButtonsItem;
var
   List:TStringlist;
begin
   result:= TGroupOptionButtonsItem(aGroup.Items.Add(TGroupOptionButtonsItem.Create));
   Result.ItemIndex:= dwIndex;

   List:= TStringList(lpzItems);
   Result.Items.AddStrings(List);
   List.Free;
end;

class function TExplorerItemFactory.Separator(aGroup: TExplorerBarGroup; dwColour: TColor): TGroupSeparatorItem;
begin
   result:= TGroupSeparatorItem(aGroup.Items.Add(TGroupSeparatorItem.Create));
   if dwColour <> -1 then Result.Color:= dwColour;
end;

class function TExplorerItemFactory.SimpleProgressBar(aGroup: TExplorerBarGroup; dwMax, dwMin, dwPosition: Integer): TGroupSimpleProgressBarItem;
begin
   result:= TGroupSimpleProgressBarItem(aGroup.Items.Add(TGroupSimpleProgressBarItem.Create));
   Result.Max:= dwMax;
   Result.Min:= dwMin;
   Result.Position:= dwPosition;
end;

class function TExplorerItemFactory.Spacer(aGroup: TExplorerBarGroup; dwHeight: Integer): TGroupSpacerItem;
begin
   result:= TGroupSpacerItem(aGroup.Items.Add(TGroupSpacerItem.Create));
   if dwHeight <> -1 then Result.Height:= dwHeight;
end;

class function TExplorerItemFactory.Text(aGroup: TExplorerBarGroup; const lpzCaption: string; aFontColour: TColor; aFontStyle: TFontStyles; Wrap: Boolean;dwIconIndex:Integer;const dwFontSize:Integer = 8;Aligned:TAlignment = taLeftJustify): TGroupTextLineItem;
begin
   result:= TGroupTextLineItem(aGroup.Items.Add(TGroupTextLineItem.Create));
   Result.Caption:= lpzCaption;
   if aFontColour > 0 then Result.FontColor:= aFontColour;
   Result.FontStyle:= aFontStyle;
   Result.WordWrap:= Wrap;
   Result.IconIndex:= dwIconIndex;
   Result.FontSize:= dwFontSize;
   if Aligned <> taLeftJustify then Result.TextAlignment:= Aligned;

end;

{ TGroupSimpleProgressBarItem }


constructor TGroupSimpleProgressBarItem.Create;
begin
   inherited;
   FMin := 0;
   FMax := 0;
   FPosition := 0;
   Height := 19;
   FBorderCOlor := clGreen;
   FFillColor := clLime;
end;

function TGroupSimpleProgressBarItem.GetItemViewerClass(Sender: TObject): TCustomBarItemViewer;
begin
   result:= TSimpleProgressBarItemViewer.Create(Self);
end;


{ TSimpleProgressBarItemViewer }

procedure TSimpleProgressBarItemViewer.Draw(Bitmap: TBitmap; var X, Y, Width: Integer);
var
   r:TRect; aItem:TGroupSimpleProgressBarItem;
begin
   aItem:= TGroupSimpleProgressBarItem(Item);
   with Bitmap.Canvas do
   begin
      r:= Rect(X + X_OFFSET, Y, X + Width - X_OFFSET, Y + Item.Height);
      Pen.Color := TGroupSimpleProgressBarItem(Item).BorderColour;
      Rectangle(X + X_OFFSET, Y, X + Width - X_OFFSET, Y + Item.Height);
      Brush.Color := aItem.FillColour;
      if (aItem.Position > 0) and (aItem.Max > 0) then
         FillRect(Rect(X + X_OFFSET + 2, Y + 2, X + X_OFFSET + 2 + Round((Width - 2 * X_OFFSET - 4) * (aItem.Position - aItem.Min) / (aItem.Max - aItem.Min)), Y + Item.Height - 2));
      Inc(Y, Item.Height + Y_OFFSET);
      Brush.Style:= bsClear;
      Item.ClientAreaRectangle:= Rect(X + X_OFFSET, Y, X + Width - X_OFFSET, Y + Item.Height);
   end;
end;

procedure TSimpleProgressBarItemViewer.Measure(Bitmap: TBitmap; var X, Y, Width: Integer);
begin
   Inc(Y, Item.Height + Y_OFFSET);
end;

{ TCustomGroupProgressBarItem }

constructor TCustomGroupProgressBarItem.Create;
begin
   inherited;
   FMin := 0;
   FMax := 0;
   FPosition := 0;
   Height := 19;
   FBorderCOlor := cl3DDkShadow;
   FFillColor := clBtnShadow;
end;

procedure TCustomGroupProgressBarItem.SetMax(Value: Integer);
begin
   FMax := Value;
   NotifyPaint;
end;

procedure TCustomGroupProgressBarItem.SetMin(Value: Integer);
begin
   FMin := Value;
   NotifyPaint;
end;

procedure TCustomGroupProgressBarItem.SetPosition(Value: Integer);
begin
   FPosition := Value;
   NotifyPaint;
end;

{ TCustomDrawProgressBarItemViewer }

procedure TCustomDrawProgressBarItemViewer.Draw(Bitmap: TBitmap; var X, Y, Width: Integer);
begin
   Item.ExplorerGroup.ExplorerBar.DoOnCustomDrawBarItem(Item,Bitmap,X,Y,Width);
end;

procedure TCustomDrawProgressBarItemViewer.Measure(Bitmap: TBitmap; var X, Y, Width: Integer);
begin
   Item.ExplorerGroup.ExplorerBar.DoOnCustomMeasureBarItem(Item,Bitmap,X,Y,Width);
end;

{ TGroupCustomDrawProgressBarItem }

function TGroupCustomDrawProgressBarItem.GetItemViewerClass(Sender: TObject): TCustomBarItemViewer;
begin
   result:= TCustomDrawProgressBarItemViewer.Create(Self);
end;

{ TCustomDrawBarItemViewer }

procedure TCustomDrawBarItemViewer.Draw(Bitmap: TBitmap; var X, Y, Width: Integer);
begin
   Item.ExplorerGroup.ExplorerBar.DoOnCustomDrawBarItem(Item,Bitmap,X,Y,Width);
end;

procedure TCustomDrawBarItemViewer.Measure(Bitmap: TBitmap; var X, Y, Width: Integer);
begin
   Item.ExplorerGroup.ExplorerBar.DoOnCustomMeasureBarItem(Item,Bitmap,X,Y,Width);
end;

{ TGroupCustomDrawBarItem }

function TGroupCustomDrawBarItem.GetItemViewerClass(Sender: TObject): TCustomBarItemViewer;
begin
   result:= TCustomDrawBarItemViewer.Create(Self);
end;

{ TGroupOptionButtonsItemViewer }

procedure TGroupOptionButtonsItemViewer.Draw(Bitmap: TBitmap; var X, Y, Width: Integer);
var
   i: Integer; ARadioRect,ACaptionRect: TRect;
begin
   with Bitmap.Canvas do
   begin
      Brush.Style:= bsClear;
      Font.Style:= [];
      ARadioRect:=   Rect(X + X_OFFSET, Y, X + X_OFFSET + Item.ExplorerGroup.ExplorerBar.ThemeElements.CheckBoxWidth , Y + Item.Height);
      ACaptionRect:= Rect(X + (X_OFFSET + X_GAP) + Item.ExplorerGroup.ExplorerBar.ThemeElements.CheckBoxWidth,Y, Width , Y + Item.Height);

      for i := 0 to TGroupOptionButtonsItem(Item).Items.Count - 1 do
      begin
         if i = TGroupOptionButtonsItem(Item).ItemIndex then
         begin
            Font.Style:= [fsBold];
            DrawRadio(Bitmap.Canvas,ARadioRect,cbchecked,True)
         end
         else
         begin
            Font.Style:= [];
            DrawRadio(Bitmap.Canvas,ARadioRect,cbUnchecked,True);
         end;
         DrawText(Handle,PChar(TGroupOptionButtonsItem(Item).Items[i]),-1,ACaptionRect,DT_SINGLELINE or DT_VCENTER or DT_LEFT or DT_END_ELLIPSIS);
         OffsetRect(ARadioRect,0,Item.ExplorerGroup.ExplorerBar.ThemeElements.CheckBoxHeight + 3);
         OffsetRect(ACaptionRect,0,Item.ExplorerGroup.ExplorerBar.ThemeElements.CheckBoxHeight + 3);
         Font.Style:= [];         
      end;
      Item.ClientAreaRectangle:= Rect(X + X_OFFSET, Y, Width - (X_OFFSET) ,Y + TGroupOptionButtonsItem(Item).Items.Count * (Item.ExplorerGroup.ExplorerBar.ThemeElements.CheckBoxHeight + 3) + (TextHeight('a')));
      Inc(Y,TextHeight('A') + LINE_Y_OFFSET);
   end;
   Inc(Y,TGroupOptionButtonsItem(Item).Items.Count * (Item.ExplorerGroup.ExplorerBar.ThemeElements.CheckBoxHeight + 3));
end;

procedure TGroupOptionButtonsItemViewer.DrawRadio(Canvas: TCanvas; R: TRect; AState: TCheckBoxState; AEnabled: Boolean);
var
   DrawRect:TRect;DrawState:Integer; element:TThemedElementDetails;
begin
   DrawRect.Left := R.Left + (R.Right - R.Left - Item.ExplorerGroup.ExplorerBar.ThemeElements.CheckBoxWidth) div 2;
   DrawRect.Top := R.Top + (R.Bottom - R.Top - Item.ExplorerGroup.ExplorerBar.ThemeElements.CheckBoxWidth) div 2;
   DrawRect.Right := DrawRect.Left + Item.Width;
   DrawRect.Bottom := DrawRect.Top + Item.Height;

   if ThemeServices.ThemesEnabled then
   begin
      case AState of
         cbChecked   :element:= ThemeServices.GetElementDetails(tbRadioButtonCheckedNormal);
         cbUnchecked :element:= ThemeServices.GetElementDetails(tbRadioButtonUncheckedNormal);
         else
            element:= ThemeServices.GetElementDetails(tbRadioButtonUncheckedNormal);
      end;
      if not AEnabled then
         element:= ThemeServices.GetElementDetails(tbRadioButtonUncheckedDisabled);
      ThemeServices.DrawElement(Canvas.Handle,element,R);
   end
   else
   begin
      case AState of
         cbChecked:
            DrawState := DFCS_BUTTONCHECK or DFCS_CHECKED;
         cbUnchecked:
            DrawState := DFCS_BUTTONCHECK;
         else
            DrawState := DFCS_BUTTON3STATE or DFCS_CHECKED;
      end;
      if not AEnabled then
         DrawState := DrawState or DFCS_INACTIVE;
      DrawFrameControl(Canvas.Handle, DrawRect, DFC_BUTTON, DrawState);
   end;
end;

procedure TGroupOptionButtonsItemViewer.Measure(Bitmap: TBitmap; var X, Y, Width: Integer);
begin
   Inc(Y,TGroupOptionButtonsItem(Item).Items.Count * (Item.ExplorerGroup.ExplorerBar.ThemeElements.CheckBoxHeight + 3));
   Inc(Y,Bitmap.Canvas.TextHeight('A') + LINE_Y_OFFSET);
end;

{ TGroupOptionButtonsItem }

constructor TGroupOptionButtonsItem.Create;
begin
   inherited;
   Items:= TStringList.Create;
   HotTracking:= True;
end;

destructor TGroupOptionButtonsItem.Destroy;
begin
   Items.Free;
   inherited;
end;

procedure TGroupOptionButtonsItem.DoMouseDown(Sender: TObject; X, Y: Integer);
var
   dwClientHeight,dwItemHeight,dwItemIndex,dwOffset:Integer;
begin
   dwClientHeight:= ClientAreaRectangle.Bottom - ClientAreaRectangle.Top;
   dwItemHeight:= (dwClientHeight div Items.Count);
   dwOffset:= Y - ClientAreaRectangle.Top;
   dwItemIndex:= (dwOffset div dwItemHeight);
   ItemIndex:= dwItemIndex;
//   OnItemClick(Self);
end;

function TGroupOptionButtonsItem.GetItemViewerClass(Sender: TObject): TCustomBarItemViewer;
begin
   result:= TGroupOptionButtonsItemViewer.Create(Self);
end;

function TGroupOptionButtonsItem.OnItemClick(Sender: TObject): Integer;
begin
   if Enabled then
   begin
      ExplorerGroup.ExplorerBar.DoOnItemClickEvent(Self,Self.ItemIndex,ExplorerGroup.Index);
   end;
end;

procedure TGroupOptionButtonsItem.SetEnabled(Value: boolean);
begin
   FEnabled:=Value;
   NotifyPaint;
end;

procedure TGroupOptionButtonsItem.set_ItemIndex(const Value: Integer);
begin
   fItemIndex := Value;
   NotifyPaint;
end;

function TGroupMenuItem.OnItemClick(Sender: TObject): Integer;
begin
   if Enabled then
   begin
      ExplorerGroup.ExplorerBar.DoOnItemClickEvent(Self,Self.Index,ExplorerGroup.Index);
   end;
end;

{ TMonthCalendarItemViewer }


procedure TMonthCalendarItemViewer.Draw(Bitmap: TBitmap; var X, Y, Width: Integer);
var
	i,p,c,spDateStartPos: Integer; DayChar:Char; dy,dm, dd, diw: Integer; r: TRect; sToday:string;
begin
   TheMonth:= MonthOf(TGroupMonthCalendarItem(Item).CalendarDate);
   TheYear:= YearOf(TGroupMonthCalendarItem(Item).CalendarDate);
   spDateStartPos:= X + 15;
   Inc(Y, Y_OFFSET);
   p := 0;
   with Bitmap.Canvas do
   begin
      Font.Color := clWindowText;
//      Pen.Color := clGrayText;
//      Rectangle(X + X_OFFSET, Y, Width - ICON_OFFSET,Y + Item.Height);
      PaintMonth(Bitmap.Canvas,GetMonthRect(bitmap,x,y,width));
      Inc(Y,TextHeight('A') + 5);
      r:= Rect(X + X_OFFSET + spDateStartPos, Y, X + X_OFFSET + 15, Y + TextHeight('A')  +5);
      Font.Color:= clWindowText;
      Brush.Style:= bsclear;
      for i := 1 to 7 do
      begin
         DayChar:= GetDayInWeekChar(i);
         TextOut(r.Left - (X_OFFSET div 2),R.Top,DayChar);
         OffsetRect(r,17,0);
      end;

	    c:= spDateStartPos;
      Inc(Y,TextHeight('A'));
      Pen.Color := clGrayText;
      MoveTo(X + X_OFFSET, Y);
      LineTo(Width - 17, Y);

      dy := TheYear;
      dm := TheMonth;
      diw := 1;
      if DayOfWeek(StartOfAMonth(TheYear, TheMonth)) > 1 then
      begin
         SetPastMonth(dy, dm);
         dd := DaysInAMonth(dy, dm) - DayOfWeek(StartOfAMonth(TheYear, TheMonth)) + 2;
      end else dd := 1;
      while p < 42 do
      begin
    	    r := Rect(c + X_OFFSET, Y, c + X_OFFSET + 15, Y + 15);
         if dm = TheMonth then Font.Color := clWindowText else Font.Color := clGrayText;
         if EncodeDate(dy, dm, dd) = TGroupMonthCalendarItem(Item).CalendarDate then
         begin
             Font.Style:= [fsbold];
             FrameRect(Rect(c + (X_OFFSET div 2), Y, c +X_OFFSET +17, Y + 15));
         end;
         if EncodeDate(dy, dm, dd) = Today then
         begin
      	     Brush.Color := clMaroon;
            FrameRect(Rect(c + (X_OFFSET div 2), Y, c + X_OFFSET + 17, Y + 15));
         end;
         PaintDay(Bitmap.Canvas,dd, r);
         Font.Style:= [];
         Inc(p);
         Inc(dd);
         if dd > DaysInAMonth(dy, dm) then
			      begin
	          SetPastMonth(dy, dm, False);
            dd := 1;
			      end;
         Inc(c, 17);
         Inc(diw);
         if diw > 7 then
         begin
            c := spDateStartPos;
            Inc(Y, 15);
            diw := 1;
         end;
      end;
      Inc(Y, Y_OFFSET);
      sToday:= FormatDateTime('dddddddd',Date);
      r:= Rect(X + X_OFFSET,Y,Width,Y + TextHeight('A'));
      Font.Color:= Item.ExplorerGroup.ExplorerBar.ColourSet.TextHot;
      DrawText(Bitmap.Canvas.Handle,PChar(sToday),-1,r,DT_SINGLELINE or DT_END_ELLIPSIS or DT_CENTER or DT_VCENTER);
      Inc(Y,TExplorerBarText.HeightOf(Bitmap.Canvas,sToday,Width));
   end;
   Inc(Y, Y_OFFSET);
end;

function TMonthCalendarItemViewer.GetDayInWeekChar(DayInWeek: Integer): Char;
begin
   Result := ' ';
   if InRange(DayInWeek, 1, 7) then Result := LongDayNames[DayInWeek][1];
end;

function TMonthCalendarItemViewer.GetMonthName(Month: Integer): String;
begin
   if InRange(Month, 1, 12) then Result := LongMonthNames[Month];
end;

function TMonthCalendarItemViewer.GetMonthRect(Bitmap: TBitmap; var X: Integer; var Y: Integer; var Width: Integer):TRect;
begin
   Result := Rect(X + X_OFFSET, Y, Width - ICON_OFFSET, Y + 16);
end;

procedure TMonthCalendarItemViewer.Measure(Bitmap: TBitmap; var X, Y, Width: Integer);
var
	i,p,c,spDateStartPos: Integer; DayChar:Char; dy,dm, dd, diw: Integer; r: TRect; sToday:string;
begin
   TheMonth:= MonthOf(TGroupMonthCalendarItem(Item).CalendarDate);
   TheYear:= YearOf(TGroupMonthCalendarItem(Item).CalendarDate);
   spDateStartPos:= X + 15;
   Inc(Y, Y_OFFSET);
   p := 0;
   with Bitmap.Canvas do
   begin
      Inc(Y,TextHeight('A') + 5);
      r:= Rect(X + X_OFFSET + spDateStartPos, Y, X + X_OFFSET + 15, Y + TextHeight('A')  +5);

	    c:= spDateStartPos;
      Inc(Y,TextHeight('A'));

      dy := TheYear;
      dm := TheMonth;
      diw := 1;
      if DayOfWeek(StartOfAMonth(TheYear, TheMonth)) > 1 then
      begin
         SetPastMonth(dy, dm);
         dd := DaysInAMonth(dy, dm) - DayOfWeek(StartOfAMonth(TheYear, TheMonth)) + 2;
      end
      else dd := 1;
      while p < 42 do
      begin
    	    r := Rect(c + X_OFFSET, Y, c + X_OFFSET + 15, Y + 15);
//         if dm = TheMonth then Font.Color := clWindowText else Font.Color := clGrayText;
         if EncodeDate(dy, dm, dd) = TGroupMonthCalendarItem(Item).CalendarDate then
         begin
             Font.Style:= [fsbold];
//             FrameRect(Rect(c + (X_OFFSET div 2), Y, c +X_OFFSET +17, Y + 15));
         end;
         if EncodeDate(dy, dm, dd) = Today then
         begin
//      	     Brush.Color := clMaroon;
//            FrameRect(Rect(c + (X_OFFSET div 2), Y, c + X_OFFSET + 17, Y + 15));
         end;
//         PaintDay(Bitmap.Canvas,dd, r);
         Font.Style:= [];
         Inc(p);
         Inc(dd);
         if dd > DaysInAMonth(dy, dm) then
			      begin
	          SetPastMonth(dy, dm, False);
            dd := 1;
			      end;
         Inc(c, 17);
         Inc(diw);
         if diw > 7 then
         begin
            c := spDateStartPos;
            Inc(Y, 15);
            diw := 1;
         end;
      end;
      Inc(Y,TextHeight('A'));
   end;
   Inc(Y,Y_OFFSET);
end;

procedure TMonthCalendarItemViewer.PaintDay(Canvas: TCanvas; Day: Integer; DayRect: TRect);
var
   StringText: string;
begin
   with Canvas.Brush do
   begin
      Style := bsClear;
      StringText := IntToStr(Day);
      Canvas.Font.Color:= clWindowText;
      DrawText(Canvas.Handle,PChar(StringText),-1,DayRect,DT_VCENTER or DT_CENTER or DT_SINGLELINE or DT_END_ELLIPSIS);
      Style := bsSolid;
   end;
end;

procedure TMonthCalendarItemViewer.PaintMonth(Canvas: TCanvas; MonthRect: TRect);
begin
   with Canvas do
   begin
      Brush.Color:= Item.ExplorerGroup.ExplorerBar.ColourSet.Background;
      FillRect(MonthRect);
      Font.Color := clWindowText;
      DrawText(Canvas.Handle,PChar(GetMonthName(TheMonth) + ' ' + IntToStr(TheYear)),-1, MonthRect,DT_VCENTER or DT_CENTER or DT_SINGLELINE or DT_END_ELLIPSIS);
   end;
end;

procedure TMonthCalendarItemViewer.SetPastMonth(var AYear, AMonth: Integer; Decrease: Boolean);
begin
	  case Decrease of
  	   True: if AMonth > 1 then AMonth := AMonth - 1 else
				     begin
				        AYear := AYear - 1;
				        AMonth := 12;
				     end;
      False:	if AMonth < 12 then AMonth := AMonth + 1 else
					    begin
					       AYear := AYear + 1;
					       AMonth := 1;
					    end;
   end;
end;

{ TGroupMonthCalendarItem }

constructor TGroupMonthCalendarItem.Create;
begin
   inherited;
//   Height:= 130;
   fCalendarDate:= Date;
end;

destructor TGroupMonthCalendarItem.Destroy;
begin

   inherited;
end;

function TGroupMonthCalendarItem.GetItemViewerClass(Sender: TObject): TCustomBarItemViewer;
begin
   result:= TMonthCalendarItemViewer.Create(Self);
end;

function TGroupMonthCalendarItem.OnItemClick(Sender: TObject): Integer;
begin

end;

procedure TGroupMonthCalendarItem.SetCalendarDate(const Value: TDateTime);
begin
   FCalendarDate := Value;
   NotifyPaint;
end;

end.
