unit codecraft.explorerbar;

interface
uses
   Windows,Forms,Classes,Contnrs,Controls,ExtCtrls,Graphics,Math,Messages,SysUtils,Types,codecraft.explorerbar.types;

type
  {flickerfree paint box OnPaint event type}
   TFFPaintEvent = procedure(Sender: TObject; Buffer: TBitmap) of object;
  {a flickerfree paint box}
   TFlickerFreePaintBox = class(TCustomControl)
   private
      FOnFFPaint: TFFPaintEvent;
      FBuffer: TBitmap; //buffered bitmap (filled when Repaint method is called)
      FDrawing: Boolean;
      FNeedRepaint: Boolean;
      FDeformHorz: Extended;
      FDeformVert: Extended;
      FDeformAlpha: Extended;
      FApplyDeform: Boolean;
   protected
      procedure Paint; override;
      procedure Resize; override;
      procedure WMEraseBkgnd(var Message: TWMEraseBkgnd); message WM_ERASEBKGND;
   public
      constructor Create(AOwner: TComponent); override;
      procedure Repaint; override;
      procedure ApplyDeform(Horz, Vert, Alpha: Extended);
      procedure NoDeform;
      property OnPaint: TFFPaintEvent read FOnFFPaint write FOnFFPaint;
   end;

   TExplorerBarGroupItem = class;

   TCustomBarItemViewerClass = class of TCustomBarItemViewer;
   TCustomBarItemViewer = class(TObject)
   private
      fItem: TExplorerBarGroupItem;
      fShowClientArea: Boolean;
   protected
      procedure DrawCaption(Bitmap: TBitmap; var X, Y:Integer; var aRect:TRect; dwFlags:Integer); virtual;
      procedure DrawIcon(Bitmap: TBitmap; var X, Y, Width: Integer); virtual;
   public
      constructor Create(aItem:TExplorerBarGroupItem); virtual;
      procedure Draw(Bitmap: TBitmap; var X, Y, Width: Integer); virtual;
      procedure Measure(Bitmap: TBitmap; var X, Y, Width: Integer); virtual;
      property Item:TExplorerBarGroupItem read fItem write fItem;
      property ShowClientArea:Boolean read fShowClientArea write fShowClientArea;
   end;

   TDefaultBarItemViewer = class(TCustomBarItemViewer)
   public
      procedure Draw(Bitmap: TBitmap; var X, Y, Width: Integer); override;
      procedure Measure(Bitmap: TBitmap; var X, Y, Width: Integer); override;
   end;

   TExplorerBarGroup = class;
  {an item - generic class for all items}
   TExplorerBarGroupItem = class(TObject)
   private
      FHeight: Integer;
      FIcon: TBitmap;
      FIconIndex: Integer;
      fIdentifier: Integer;
      FState: TBarItemState;
      FHint: string;
      fWidth: Integer;
      FCaption: string;
      FEnabled: boolean;
      fExplorerGroup: TExplorerBarGroup;
      fIndex: Integer;
      fHotTracking: Boolean;
      FMouseInControl: Boolean;
      fWordWrap: Boolean;
      fFontStyle: TFontStyles;
      FFontColor: TColor;
      procedure Draw(Bitmap: TBitmap; var X, Y, Width: Integer); virtual;
      procedure Measure(Bitmap: TBitmap; var X, Y, Width: Integer); virtual;
      procedure SetHeight(const Value: Integer);
      procedure SetIcon(const Value: TBitmap);
      procedure SetIconIndex(const Value: Integer);
      procedure SetState(const Value: TBarItemState); virtual;
      function get_ExplorerGroup: TExplorerBarGroup;
      procedure SetCaption(const Value: string);
      procedure SetMouseInControl(const Value: Boolean); virtual;
      procedure SetFontColor(const Value: TColor);
      function get_ItemRectangle: TRect;
   protected
      procedure SetEnabled(const Value: boolean); virtual;
      procedure DoMouseDown(Sender:TObject;X,Y:Integer); virtual;      
   public
      ClientAreaRectangle:TRect;
      procedure NotifyPaint;
      constructor Create; overload; virtual;
      destructor Destroy; override;
      function GetItemViewerClass(Sender:TObject):TCustomBarItemViewer; virtual; abstract;
      function OnItemClick(Sender:TObject):Integer; virtual;
      property MouseInControl:Boolean read FMouseInControl write SetMouseInControl;
      property ExplorerGroup: TExplorerBarGroup read fExplorerGroup write fExplorerGroup;
      property Height:Integer read FHeight write SetHeight;
      property Width:Integer read fWidth write fWidth;
      property Icon:TBitmap read FIcon write SetIcon;
      property IconIndex:Integer read FIconIndex write SetIconIndex;
      property Identifier:Integer read fIdentifier write fIdentifier;
      property State:TBarItemState read FState write SetState;
      property Hint: string read FHint write FHint;
      property Caption: string read FCaption write SetCaption;
      property Enabled: boolean read FEnabled write SetEnabled;
      property Index:Integer read fIndex write fIndex;
      property HotTracking:Boolean read fHotTracking write fHotTracking;
      property WordWrap:Boolean read fWordWrap write fWordWrap;
      property FontColor: TColor read FFontColor write SetFontColor;
      property FontStyle:TFontStyles read fFontStyle write fFontStyle;
      property ItemRectangle:TRect read get_ItemRectangle;

   end;

   TExplorerBarGroupItems = class(TObjectList)
   private
      fGroup: TExplorerBarGroup;
      function GetItem(Index: Integer): TExplorerBarGroupItem;
      procedure SetItem(Index: Integer; Value: TExplorerBarGroupItem);
   protected
   public
      constructor Create(fGroup:TExplorerBarGroup);
      destructor Destroy; override;
      function Add(aItem:TExplorerBarGroupItem):TExplorerBarGroupItem;
      function Remove(aItem: TExplorerBarGroupItem): Integer;
      function IndexOf(aItem: TExplorerBarGroupItem): Integer;
      property Items[Index: Integer]: TExplorerBarGroupItem read GetItem write SetItem; default;
      property Group:TExplorerBarGroup read fGroup write fGroup;
   end;

   TExplorerBar = class;
  {a group of items - clickable}
   TExplorerBarGroup = class(TCollectionItem)
   private
      FAnimStep: Integer;
      FNbSteps: Integer;
      FDestHeight: Integer;
      FShownHeight: Integer;
      FBgImage: TBitmap;
      FGroupIconIndex: Integer;
      FItems: TExplorerBarGroupItems;
      FSpecialGroup: Boolean;
      FState: TGroupState;
      FTimer: TTimer;
      FTitle: string;
      FTitleRect: TRect;
      FBodyRect: TRect;
      FHeight: Integer;
      FWidth: Integer;
      procedure Draw(Bitmap: TBitmap; var X, Y: Integer);
      procedure Measure(Bitmap: TBitmap; var X, Y: Integer); overload;
      procedure Measure; overload;
      procedure SetBgImage(Value: TBitmap);
      procedure SetGroupIconIndex(Value: Integer);
      procedure SetSpecialGroup(Value: Boolean);
      procedure SetTitle(Value: string);
      procedure Timer(Sender: TObject);
      function get_ExplorerBar: TExplorerBar;
   public
      constructor Create(Collection:TCollection); override;
      destructor Destroy; override;
      procedure OpenClose;
      procedure Collapse;
      procedure Expand;
      function FindItem(const lpzTitle,lpzClassName:string):TExplorerBarGroupItem;
      property Items: TExplorerBarGroupItems read FItems write fItems;
      property ExplorerBar:TExplorerBar read get_ExplorerBar;
      property Width:Integer read fWidth;
   published
      property BgImage: TBitmap read FBgImage write SetBgImage;
      property SpecialGroup: Boolean read FSpecialGroup write SetSpecialGroup;
      property State: TGroupState read FState write FState;
      property Title: string read FTitle write SetTitle;
      property GroupIconIndex: Integer read FGroupIconIndex write SetGroupIconIndex;
   end;

  {a list of groups}
   TExplorerBarGroups = class(TCollection)
   private
      FOwner: TExplorerBar;
      function GetItem(Index: Integer): TExplorerBarGroup;
      procedure SetItem(Index: Integer; Value: TExplorerBarGroup);
   public
      constructor Create(AOwner: TComponent);
      destructor Destroy; override;
      function AddTitle(const ATitle: string; ASpecialGroup: Boolean = False): TExplorerBarGroup;
      function Add: TExplorerBarGroup;
      property ExplorerBar:TExplorerBar read fOwner write fOwner;
      property Items[Index: Integer]: TExplorerBarGroup read GetItem write SetItem; default;
   published
   end;

   TExplorerBarColourSet = class(TObject)
   private
      fText: TColor;
      fTextHot: TColor;
      fBar: TExplorerBar;
      fGroupBackground: TColor;
      fBackground: TColor;
      fSpecialText: TColor;
      fSpecialTextHot: TColor;
      fBorder: TColor;
   protected
   public
      constructor Create(aBar:TExplorerBar);
      procedure SavetoFile(const lpzFilename:string);
      procedure LoadFromFile(const lpzFilename,lpzThemeName:string);
      property TextHot:TColor read fTextHot write fTextHot;
      property Text:TColor read fText write fText;
      property Border:TColor read fBorder write fBorder;
      property Background:TColor read fBackground write fBackground;
      property GroupBackground:TColor read fGroupBackground write fGroupBackground;
      property SpecialText:TColor read fSpecialText write fSpecialText;
      property SpecialTextHot:TColor read fSpecialTextHot write fSpecialTextHot;
      property Bar:TExplorerBar read fBar write fBar;
   end;

   TExplorerBarThemeElements = class(TObject)
   private
      fOliveElements: TBitmap;
      fBlueElements: TBitmap;
      fBar: TExplorerBar;
      fSilverElements: TBitmap;
      fArrowDown: TBitmap;
      fArrowUp: TBitmap;
      fCheckBoxWidth: Integer;
      fCheckBoxHeight: Integer;
      fThemeNames: TStringList;
      FTheme: String;
      fOrangeElements: TBitmap;
      fBlueFlatElements: TBitmap;
      function get_Height: Integer;
      function get_Theme: TBitmap;
      procedure SetThemeName(const Value: String);
   protected
      procedure set_CheckBoxDimensions;
      procedure DrawMask(Dest: TBitmap; X, Y: Integer; Src: TBitmap);
      function ElementRect(dwType:TThemeElementRectType):TRect;
      property BlueElements:TBitmap read fBlueElements write fBlueElements;
      property SilverElements:TBitmap read fSilverElements write fSilverElements;
      property OliveElements:TBitmap read fOliveElements write fOliveElements;
      property OrangeElements:TBitmap read fOrangeElements write fOrangeElements;
      property BlueFlatElements:TBitmap read fBlueFlatElements write fBlueFlatElements;
      property ArrowUp:TBitmap read fArrowUp write fArrowUp;
      property ArrowDown:TBitmap read fArrowDown write fArrowDown;
   public
      constructor Create(aBar:TExplorerBar);
      destructor Destroy; override;
      procedure DrawThemedHeader(Bitmap, Skin: TBitmap; X, Y, Width: Integer; SpecialGroup, Closed, MouseOver: Boolean);
      procedure DrawHeader(Bitmap: TBitmap; X, Y, Width: Integer; SpecialGroup, Closed: Boolean; TextColor: TColor);
      procedure BlendFillRect(Dest: TBitmap; ARect: TRect; Alpha: Extended = 1);
      property Bar:TExplorerBar read fBar write fBar;
      property CheckBoxWidth:Integer read fCheckBoxWidth write fCheckBoxWidth;
      property CheckBoxHeight:Integer read fCheckBoxHeight write fCheckBoxHeight;
      property Height:Integer read get_Height;
      property Theme:TBitmap read get_Theme;
      property ThemeNames:TStringList read fThemeNames write fThemeNames;
   published
      property ThemeName:String read fTheme write SetThemeName;
   end;

  {a complete bar}
   TExplorerBar = class(TScrollingWinControl)
   private
      FAnimStep: Integer;
      FUpdating: Boolean;
      FAnimate: Boolean;
      fGroups: TExplorerBarGroups;
      FGroupIcons: TImageList;
      FHotArea: TObject;
      FItemIcons: TImageList;
      FNeedResize: Boolean;
      FOnClick: TMenuItemClickEvent;
      FPaintBox: TFlickerFreePaintBox;
      FTimer: TTimer;
      FOldWidth: Integer;
      FGroupsHeight: Integer;
      fHoverGroup: TExplorerBarGroup;
      fColourSet: TExplorerBarColourSet;
      fMousePosition: TMouseHotTrackPosition;
      fOnItemClick: TOnItemClickEvent;
      fThemeElements: TExplorerBarThemeElements;
      fOnBeforeGroupPaint: TNotifyEvent;
      fOnBeforePaint: TNotifyEvent;
      fOnAfterDrawGroup: TNotifyEvent;
      fOnAfterDraw: TNotifyEvent;
      fOnCustomMeasureBarItem: TOnCustomMeasureBarItem;
      fOnCustomDrawBarItem: TOnCustomDrawBarItem;
      procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
      function GetHotItem(X, Y: Integer):TObject;
      procedure PaintBuffer(Sender: TObject; Buffer: TBitmap);
      procedure PbMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
      procedure PbMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
      procedure PbMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
      procedure SetGroupIcons(Value: TImageList);
      procedure Measure;
      procedure SetItemIcons(Value: TImageList);
      procedure Timer(Sender: TObject);
   protected
      procedure Resize; override;
   public
      constructor Create(AOwner: TComponent); override;
      destructor Destroy; override;
      procedure DoOnItemClickEvent(Item:TObject; ItemIndex,GroupIndex:Integer);
      procedure DoOnCustomDrawBarItem(BarItem:TObject;Bitmap: TBitmap; var X: Integer; var Y: Integer; var Width: Integer);
      procedure DoOnCustomMeasureBarItem(BarItem:TObject; Bitmap: TBitmap; var X: Integer; var Y: Integer; var Width: Integer);
      procedure BeginUpdate;
      procedure EndUpdate;
      procedure Repaint; override;
      procedure ShowAnimate;
      procedure FocusToGroup(dwIndex:Integer);
      procedure CloseAllGroups;

      property HoverGroup:TExplorerBarGroup read fHoverGroup write fHoverGroup;
      property ColourSet:TExplorerBarColourSet read fColourSet write fColourSet;
      property MousePosition:TMouseHotTrackPosition read fMousePosition write fMousePosition;
      property HotArea:TObject read fHotArea write fHotArea;
      property Updating:Boolean read FUpdating write FUpdating;
   published
      property Animate: Boolean read FAnimate write FAnimate;
      property GroupIcons: TImageList read FGroupIcons write SetGroupIcons;
      property ItemIcons: TImageList read FItemIcons write SetItemIcons;
      property OnMenuItemClick: TMenuItemClickEvent read FOnClick write FOnClick;
      property OnItemClick:TOnItemClickEvent read fOnItemClick write fOnItemClick;
      property OnBeforeDrawGroup:TNotifyEvent read fOnBeforeGroupPaint write fOnBeforeGroupPaint;
      property OnBeforeDraw:TNotifyEvent read fOnBeforePaint write fOnBeforePaint;
      property OnAfterDraw:TNotifyEvent read fOnAfterDraw write fOnAfterDraw;
      property OnAfterDrawGroup:TNotifyEvent read fOnAfterDrawGroup write fOnAfterDrawGroup;
      property OnCustomDrawBarItem:TOnCustomDrawBarItem read fOnCustomDrawBarItem write fOnCustomDrawBarItem;
      property OnCustomMeasureBarItem:TOnCustomMeasureBarItem read fOnCustomMeasureBarItem write fOnCustomMeasureBarItem;
      property Groups: TExplorerBarGroups read fGroups;
      property ThemeElements:TExplorerBarThemeElements read fThemeElements write fThemeElements;
      property Constraints;
      property Enabled;
      property Font;
      property ParentFont;
      property PopupMenu;
      property ShowHint;
      property TabOrder;
      property Visible;
      property OnCanResize;
      property OnClick;
      property OnConstrainedResize;
      property OnContextPopup;
      property OnDblClick;
      property OnEnter;
      property OnExit;
      property OnGetSiteInfo;
      property OnMouseDown;
      property OnMouseMove;
      property OnMouseUp;
      property OnMouseWheel;
      property OnMouseWheelDown;
      property OnMouseWheelUp;
      property OnResize;
      property HorzScrollBar;
      property VertScrollBar;
   end;

implementation
uses
   INIFiles,codecraft.explorerbar.utils, codecraft.explorerbar.consts;

{$R C:\Packages\Common\Codecraft\Resources\explorerBarRes.res}

{-----------------------------------------------
resize a bitmap}

procedure ResizeBmp(var Src, Dst: TBitmap);
begin
   SetStretchBltMode(Dst.Canvas.Handle, HALFTONE);
   Dst.Canvas.CopyRect(Dst.Canvas.ClipRect, Src.Canvas, Src.Canvas.ClipRect);
end;

{-----------------------------------------------
nb of steps to change height from H1 to H2}

function CalcNbSteps(H1, H2, Incr: Integer): Integer;
var
   NbSteps: Integer;
   Buf: Integer;
begin
   NbSteps := 1;
   if H1 > H2 then
   begin
      Buf := H1;
      H1 := H2;
      H2 := Buf;
   end;
   while H1 < H2 do
   begin
      Inc(H1, NbSteps);
      Inc(NbSteps, Incr);
   end;
   Result := NbSteps;
end;

{------------------------------------------------
 TFlickerFreePaintBox
------------------------------------------------}

procedure TFlickerFreePaintBox.Paint;
var
   Bmp: TBitmap;
begin
   if csDesigning in ComponentState then
      with Canvas do
      begin
         Pen.Style := psDash;
         Brush.Style := bsSolid;
         Canvas.Brush.Color := Color;
         Rectangle(0, 0, Width, Height);
         Exit;
      end;

   if FNeedRepaint then
      Repaint
   else if FApplyDeform then
      begin
         FDrawing := True;
         Bmp := TBitmap.Create;
         try
            Bmp.PixelFormat := pf24bit;
            Bmp.Width := FBuffer.Width;
            Bmp.Height := FBuffer.Height;
            TExplorerBarRenderer.DrawAlpha(Bmp, Color, 0, 0, FBuffer.Canvas.ClipRect, FBuffer, FDeformAlpha);
            Canvas.CopyRect(Rect(0, 0, Round(FBuffer.Width * FDeformHorz), Round(FBuffer.Height * FDeformVert)), Bmp.Canvas, Bmp.Canvas.ClipRect);
            Canvas.Brush.Color := Color;
            Canvas.FillRect(Rect(Round(FBuffer.Width * FDeformHorz), 0, Width - 1, Height - 1));
            Canvas.FillRect(Rect(0, Round(FBuffer.Height * FDeformVert), Width - 1, Height - 1));
         finally
            Bmp.Free;
            FDrawing := False;
         end;
      end
      else
      begin
         FDrawing := True;
         Canvas.Draw(0, 0, FBuffer);
         FDrawing := False;
      end;
end;

{-----------------------------------------------}

procedure TFlickerFreePaintBox.Resize;
begin
   if FDrawing then
      Exit;
   FBuffer.Free;
   FBuffer := TBitmap.Create;
   FBuffer.PixelFormat := pf24bit;
   FBuffer.Width := Width;
   FBuffer.Height := Height;
   FBuffer.Canvas.Brush.Color := Color;
   FNeedRepaint := True;
end;

{-----------------------------------------------}

procedure TFlickerFreePaintBox.WMEraseBkgnd(var Message: TWMEraseBkgnd);
begin
   Message.Result := 1;
end;

{-----------------------------------------------}

constructor TFlickerFreePaintBox.Create(AOwner: TComponent);
begin
   inherited Create(AOwner);
   TabStop := False;
   FBuffer := TBitmap.Create;
   FBuffer.Width := Width;
   FBuffer.Height := Height;
   FBuffer.PixelFormat := pf24bit;
   FBuffer.Canvas.Brush.Color := Color;
   FNeedRepaint := True;
   FDrawing := False;
end;

{-----------------------------------------------}

procedure TFlickerFreePaintBox.Repaint;
begin
   if csDesigning in ComponentState then
      Exit;

   FBuffer.Canvas.FillRect(FBuffer.Canvas.ClipRect);
   if Assigned(FOnFFPaint) then
      FOnFFPaint(Self, FBuffer);
   FNeedRepaint := False;
   Paint;
end;

{-----------------------------------------------}

procedure TFlickerFreePaintBox.ApplyDeform(Horz, Vert, Alpha: Extended);
begin
   FApplyDeform := True;
   FDeformHorz := Horz;
   FDeformVert := Vert;
   FDeformAlpha := Alpha;
   Paint;
end;

{-----------------------------------------------}

procedure TFlickerFreePaintBox.NoDeform;
begin
   FApplyDeform := False;
   Paint;
end;

{------------------------------------------------
 TExCustomItem
------------------------------------------------}

destructor TExplorerBarGroupItem.Destroy;
begin
   if fIcon <> nil then fIcon.Free;
   inherited;
end;

procedure TExplorerBarGroupItem.DoMouseDown(Sender: TObject; X, Y: Integer);
begin

end;

procedure TExplorerBarGroupItem.Draw(Bitmap: TBitmap; var X, Y, Width: Integer);
begin
   GetItemViewerClass(Self).Draw(Bitmap,X,Y,Width);
end;

function TExplorerBarGroupItem.get_ExplorerGroup: TExplorerBarGroup;
begin
   result:= fExplorerGroup;
end;

function TExplorerBarGroupItem.get_ItemRectangle: TRect;
begin
   result:= ClientAreaRectangle;
end;

{-----------------------------------------------}

procedure TExplorerBarGroupItem.Measure(Bitmap: TBitmap; var X, Y, Width: Integer);
begin
   GetItemViewerClass(Self).Measure(Bitmap,X,Y,Width);
end;

procedure TExplorerBarGroupItem.NotifyPaint;
begin
   if not ExplorerGroup.ExplorerBar.FUpdating then ExplorerGroup.ExplorerBar.Repaint;
end;

function TExplorerBarGroupItem.OnItemClick(Sender: TObject): Integer;
begin
end;


procedure TExplorerBarGroupItem.SetCaption(const Value: string);
begin
   FCaption := Value;
   NotifyPaint;
end;

procedure TExplorerBarGroupItem.SetEnabled(const Value: boolean);
begin
   FEnabled := Value;
   NotifyPaint;
end;

procedure TExplorerBarGroupItem.SetFontColor(const Value: TColor);
begin
   FFontColor := Value;
end;

procedure TExplorerBarGroupItem.SetHeight(const Value: Integer);
begin
   FHeight := Value;
end;

procedure TExplorerBarGroupItem.SetIcon(const Value: TBitmap);
begin
   if Value = nil then
   begin
      if FIcon <> nil then
      begin
         FIcon.Free;
         FIcon := nil;
      end;
   end
   else
   begin
      if FIcon = nil then
         FIcon := TBitmap.Create;
      FIcon.Assign(Value);
   end;
   NotifyPaint;
end;

procedure TExplorerBarGroupItem.SetIconIndex(const Value: Integer);
begin
   FIconIndex:= value;
end;

procedure TExplorerBarGroupItem.SetMouseInControl(const Value: Boolean);
begin
   FMouseInControl := Value;
end;

procedure TExplorerBarGroupItem.SetState(const Value: TBarItemState);
begin
   FState := Value;
   NotifyPaint;
end;

constructor TExplorerBarGroupItem.Create;
begin
   inherited;
   FIcon := nil;
   FIconIndex := UNKNOWN_VALUE;
   fIdentifier := UNKNOWN_VALUE;
   ClientAreaRectangle := UNKNOWN_RECT;
   fEnabled:= True;
   FHeight:= DEFAULT_ITEM_HEIGHT;
   FFontColor:= clWindowText;
   fFontStyle:= [];
end;

procedure TExplorerBarGroup.Draw(Bitmap: TBitmap; var X, Y: Integer);
var
   I: Integer;BodyRgn:HRGN; TitleRect:TRect;
begin
   if ExplorerBar.FUpdating then Exit;
   BodyRgn := CreateRectRgn(X, Y, X + FWidth, Y + ICON_SIDE);
   SelectClipRgn(Bitmap.Canvas.Handle, BodyRgn);
   ExplorerBar.ThemeElements.DrawThemedHeader(Bitmap, ExplorerBar.ThemeElements.Theme, X, Y, FWidth, FSpecialGroup, FState in [gsClosed, gsClosing], Self = ExplorerBar.FHotArea);
//   else
//      ExplorerBar.ThemeElements.DrawHeader(Bitmap, X, Y, FWidth, FSpecialGroup, FState in [gsClosed, gsClosing], IfThen(FSpecialGroup, IfThen(Self <> ExplorerBar.FHotArea, ExplorerBar.ColourSet.SpecialText, ExplorerBar.ColourSet.SpecialTextHot), IfThen(Self <> ExplorerBar.FHotArea, ExplorerBar.ColourSet.Text, ExplorerBar.ColourSet.TextHot)));
  //draw icon and write title
   with Bitmap.Canvas do
   begin
      Brush.Style := bsClear;
      Font.Style := [fsBold];
      Font.Color := IfThen(FSpecialGroup,
         IfThen(Self <> ExplorerBar.FHotArea, ExplorerBar.ColourSet.SpecialText, ExplorerBar.ColourSet.SpecialTextHot),
         IfThen(Self <> ExplorerBar.FHotArea, ExplorerBar.ColourSet.Text, ExplorerBar.ColourSet.TextHot));
      if (FGroupIconIndex <> UNKNOWN_VALUE) and (ExplorerBar.FGroupIcons <> nil) then
      begin
         ExplorerBar.FGroupIcons.Draw(Bitmap.Canvas, X, Y, FGroupIconIndex);
         TitleRect:= Rect(X + 2 * ICON_OFFSET + ICON_SIDE,Y + Y_OFFSET + (ICON_SIDE - Y_OFFSET - TextHeight('A')) div 2 , X + FWidth - SKIN_RIGHT_PART_WIDTH, Y + ICON_SIDE);
         DrawText(Handle,PChar(Title),-1,TitleRect,DT_LEFT or DT_VCENTER or DT_SINGLELINE or DT_END_ELLIPSIS);
//         TextRect(Rect(X, Y, X + FWidth - SKIN_RIGHT_PART_WIDTH, Y + ICON_SIDE), X + 2 * ICON_OFFSET + ICON_SIDE, Y + Y_OFFSET + (ICON_SIDE - Y_OFFSET - TextHeight('A')) div 2, FTitle);
      end
      else
      begin
         TitleRect:= Rect(X + 2 * ICON_OFFSET ,Y + Y_OFFSET + (ICON_SIDE - Y_OFFSET - TextHeight('A')) div 2 , X + FWidth - SKIN_RIGHT_PART_WIDTH, Y + ICON_SIDE);
         DrawText(Handle,PChar(Title),-1,TitleRect,DT_LEFT or DT_VCENTER or DT_SINGLELINE or DT_END_ELLIPSIS);
//         TextRect(Rect(X, Y, X + FWidth - SKIN_RIGHT_PART_WIDTH, Y + ICON_SIDE), X + 2 * ICON_OFFSET, Y + Y_OFFSET + (ICON_SIDE - Y_OFFSET - TextHeight('A')) div 2, FTitle);
      end;

   end;
   DeleteObject(BodyRgn);
   FTitleRect := Rect(X, Y + ICON_SIDE - ExplorerBar.ThemeElements.Height , X + FWidth, Y + ICON_SIDE);
   Inc(Y, ICON_SIDE);
   FBodyRect.TopLeft := Point(X, Y);
   FBodyRect.Right := X + FWidth;
   if FState <> gsClosed then
   begin
      BodyRgn := CreateRectRgn(X, Y, X + FWidth, Y + FShownHeight);
      SelectClipRgn(Bitmap.Canvas.Handle, BodyRgn);
    //draw body background
      with Bitmap.Canvas do
      begin
         Brush.Color := ExplorerBar.ColourSet.GroupBackground;
         Brush.Style := bsSolid;
         FillRect(Bitmap.Canvas.ClipRect);
      //draw BgImage
         if FBgImage <> nil then Draw(X + FWidth - FBgImage.Width, Y + FShownHeight - FBgImage.Height, FBgImage);
         Pen.Color := ExplorerBar.ColourSet.Border;
         Pen.Style := psSolid;
         MoveTo(X, Y);
         LineTo(X, Y + FShownHeight - 1);
         LineTo(X + FWidth - 1, Y + FShownHeight - 1);
         LineTo(X + FWidth - 1, Y);
      end;
    //draw items
      if FState in [gsOpening, gsCLosing] then Y := Y - FHeight + FShownHeight;
      Inc(Y, Y_OFFSET);
      for I := 0 to FItems.Count - 1 do
         try
            (FItems[I] as TExplorerBarGroupItem).Draw(Bitmap, X, Y, FWidth);
         except
         end;
      Inc(Y, Y_OFFSET);

      if FState in [gsOpening, gsCLosing] then
      begin
         Bitmap.Canvas.Brush.Color := ExplorerBar.ColourSet.Background;
         ExplorerBar.ThemeElements.BlendFillRect(Bitmap, Bitmap.Canvas.ClipRect, 1 - FShownHeight / FHeight);
      end;
      DeleteObject(BodyRgn);
   end;
   FBodyRect.Bottom := Y;
   Inc(Y, Y_OFFSET);
end;

procedure TExplorerBarGroup.Expand;
begin
   Self.State:= gsClosed;
   Self.OpenClose;
end;

function TExplorerBarGroup.FindItem(const lpzTitle, lpzClassName: string): TExplorerBarGroupItem;
var
   i:Integer; aITem:TExplorerBarGroupItem;
begin
   result:= nil;
   for i := 0 to Items.Count - 1 do
   begin
      aItem:= Items.GetItem(i);
      if (CompareText(aItem.ClassName,lpzClassName) = 0) and (CompareText(aITem.Caption,lpzTitle) = 0) then
      begin
         result:= aITem;
         Break;
      end;
   end;
end;

function TExplorerBarGroup.get_ExplorerBar: TExplorerBar;
begin
   result:= TExplorerBarGroups(Collection).ExplorerBar;
end;

{-----------------------------------------------}

procedure TExplorerBarGroup.Measure(Bitmap: TBitmap; var X, Y: Integer);
var
   I, Bottom: Integer;
begin
   if ExplorerBar.FUpdating then Exit;
   Inc(Y, ICON_SIDE);
   Bottom := Y;
   Inc(Bottom, Y_OFFSET);
   for I := 0 to Items.Count - 1 do
   begin
      Items[I].Measure(Bitmap, X, Bottom, FWidth);
   end;

   Inc(Bottom, Y_OFFSET);
   FHeight := Bottom - Y;
   if (FShownHeight > FHeight) or (FState = gsOpen) then
      FShownHeight := FHeight;
   if FState in [gsOpen, gsOpening] then
      Y := Bottom;
   Inc(Y, Y_OFFSET);
end;

{-----------------------------------------------}

procedure TExplorerBarGroup.Measure;
var
   X, Y: Integer;
begin
   X := 0;
   Y := 0;
   Measure(ExplorerBar.FPaintBox.FBuffer, X, Y);
end;

{-----------------------------------------------}

procedure TExplorerBarGroup.SetBgImage(Value: TBitmap);
begin
   if Value <> nil then
   begin
      if FBgImage = nil then
         FBgImage := TBitmap.Create;
      FBgImage.Assign(Value);
   end
   else if FBgImage <> nil then
      begin
         FBgImage.Free;
         FBgImage := nil;
      end;
end;

{-----------------------------------------------}

procedure TExplorerBarGroup.SetGroupIconIndex(Value: Integer);
begin
   FGroupIconIndex := Value;
   if not ExplorerBar.FUpdating then
      ExplorerBar.Repaint;
end;

{-----------------------------------------------}

procedure TExplorerBarGroup.SetSpecialGroup(Value: Boolean);
begin
   FSpecialGroup := Value;
   if not ExplorerBar.FUpdating then
      ExplorerBar.Repaint;
end;

{-----------------------------------------------}

procedure TExplorerBarGroup.SetTitle(Value: string);
begin
   FTitle := Value;
   if not ExplorerBar.FUpdating then
      ExplorerBar.Repaint;
end;

{-----------------------------------------------}

procedure TExplorerBarGroup.Timer(Sender: TObject);
begin
   case FState of
      gsOpening:
      begin
         FShownHeight := FShownHeight + FAnimStep;
         if FShownHeight > FHeight then
            FShownHeight := FHeight;
         Dec(FAnimStep, 1 + FHeight div 150);
      end;
      gsClosing:
      begin
         FShownHeight := FShownHeight - FAnimStep;
         if FShownHeight < 0 then
            FShownHeight := 0;
         Inc(FAnimStep, 1 + FHeight div 150);
      end;
   end;
   if (FAnimStep >= FNbSteps) or (FAnimStep < 0) then
   begin
      FShownHeight := FDestHeight;
      FTimer.Enabled := False;
      if FShownHeight = 0 then
         FState := gsClosed
      else
         FState := gsOpen;
   end;
   ExplorerBar.Measure;
   ExplorerBar.Repaint;
end;

{-----------------------------------------------}

procedure TExplorerBarGroup.Collapse;
begin
   Self.State:= gsOpen;
   Self.OpenClose;
end;

constructor TExplorerBarGroup.Create(Collection:TCollection);
begin
   inherited Create(Collection);
   FItems := TExplorerBarGroupItems.Create(Self);
   FState := gsOpen;
   FTimer := TTimer.Create(nil);
   FTimer.Enabled := False;
   FTimer.Interval := ANIM_SPEED;
   FTimer.OnTimer := Timer;
   FHeight := 25;
   FDestHeight := FHeight;
   FShownHeight := FHeight;
   FWidth := ExplorerBar.ClientWidth - 2 * X_OFFSET;
   FGroupIconIndex := UNKNOWN_VALUE;
   FBgImage := nil;
end;

{-----------------------------------------------}

destructor TExplorerBarGroup.Destroy;
begin {override}
   if ExplorerBar.FHotArea = Self then ExplorerBar.FHotArea := nil;
   FItems.Free;
   FTimer.Free;
   inherited;
end;

{-----------------------------------------------}

procedure TExplorerBarGroup.OpenClose;
var
   DoCalc: Boolean;
begin
   if not ExplorerBar.FAnimate then
   begin
      case FState of
         gsOpen:
            FState := gsClosed;
         gsClosed:
            FState := gsOpen;
      end;
   end
   else
   begin
      DoCalc := False;
      case FState of
         gsOpen:
         begin
            DoCalc := True;
            FDestHeight := 0;
            FShownHeight := FHeight;
            FState := gsClosing;
            FTimer.Enabled := True;
         end;
         gsClosed:
         begin
            DoCalc := True;
            FDestHeight := FHeight;
            FShownHeight := 0;
            FState := gsOpening;
            FTimer.Enabled := True;
         end;
         gsOpening:
         begin
            FDestHeight := 0;
            FState := gsClosing;
         end;
         gsClosing:
         begin
            FDestHeight := FHeight;
            FState := gsOpening;
         end;
      end;
      if FShownHeight > FHeight then
         FShownHeight := FHeight;
      if DoCalc then
      begin
         FNbSteps := CalcNbSteps(0, FHeight, 1 + FHeight div 150);
         if FState = gsOpening then
            FAnimStep := FNbSteps - 1
         else
            FAnimStep := 0;
      end;
   end;
   ExplorerBar.Measure;
   ExplorerBar.Repaint;
end;

{------------------------------------------------
 TExGroupList
------------------------------------------------}

function TExplorerBarGroups.Add: TExplorerBarGroup;
begin
   Result := TExplorerBarGroup(inherited Add);
end;

function TExplorerBarGroups.AddTitle(const ATitle: string; ASpecialGroup: Boolean = False): TExplorerBarGroup;
begin
   result:= Add;
   Result.Title:= ATitle;
//   ExGroup := TExplorerGroup.Create(FOwner);
   Result.SpecialGroup:= ASpecialGroup;
//   inherited Add(ExGroup);
   FOwner.FNeedResize := True;
end;

{-----------------------------------------------}

constructor TExplorerBarGroups.Create(AOwner: TComponent);
begin
   inherited Create(TExplorerBarGroup);
   FOwner := TExplorerBar(AOwner);
end;

{-----------------------------------------------}

destructor TExplorerBarGroups.Destroy;
begin
   inherited;
end;

{-----------------------------------------------}

function TExplorerBarGroups.GetItem(Index: Integer): TExplorerBarGroup;
begin
   Result := TExplorerBarGroup(inherited GetItem(Index));
end;

{-----------------------------------------------}

procedure TExplorerBarGroups.SetItem(Index: Integer; Value: TExplorerBarGroup);
begin
   inherited SetItem(Index, Value);
end;

procedure TExplorerBar.CMMouseLeave(var Message: TMessage);
var
   OldHotArea: TObject;DrawX, DrawY: Integer;
begin
   if (HotArea <> nil) then
   begin
      OldHotArea := HotArea;
      HotArea := nil;
      if (OldHotArea <> nil) and (OldHotArea is TExplorerBarGroupItem) then OldHotArea := (OldHotArea as TExplorerBarGroupItem).ExplorerGroup;
      if OldHotArea <> nil then
         try
            with OldHotArea as TExplorerBarGroup do
            begin
               DrawX := FTitleRect.Left;
               DrawY := FTitleRect.Top - ICON_SIDE + ThemeElements.Height;
               Draw(FPaintBox.FBuffer, DrawX, DrawY);
            end;
         except
         end;
      FPaintBox.Paint;
   end;
end;

function TExplorerBar.GetHotItem(X, Y: Integer): TObject;
var
   I, J: Integer;
   Found: Boolean;
begin
   Result := nil;
   MousePosition:= mhtNone;
   Found := False;
   I := 0;
   while not Found and (I < Groups.Count) do
      with Groups[I] do
      begin
         if PtInRect(FTitleRect, Point(X, Y)) then
         begin
            MousePosition:= mhtTitle;
            Result := Groups[I];
            Found := True;
         end
         else if PtInRect(FBodyRect, Point(X, Y)) then
            begin
               J := 0;
               MousePosition:= mhtBody;
               while not Found and (J < Items.Count) do
               begin
                  if (PtInRect((Items[J]).ClientAreaRectangle, Point(X, Y))) and (Items[J].HotTracking) and (Items[j].Enabled) then
                  begin
                     MousePosition:= mhtItem;
                     Result := Items[J];
                     Items[j].State:= [bisHot];
                     Items[j].MouseInControl:= True;
                     Found := True;
                  end;
                  Inc(J);
               end;
            end;
         Inc(I);
      end;
end;

{-----------------------------------------------}

procedure TExplorerBar.PbMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
var
   NewHotArea: TObject; OldHotArea: TObject; DrawX, DrawY: Integer;
begin
   if Shift <> [] then Exit;
   if not ((HotArea <> nil) and
      ((FHotArea is TExplorerBarGroupItem) and PtInRect((FHotArea as TExplorerBarGroupItem).ClientAreaRectangle, Point(X, Y))) or
      ((FHotArea is TExplorerBarGroup) and PtInRect((FHotArea as TExplorerBarGroup).FTitleRect, Point(X, Y)))) then
   begin
      NewHotArea := GetHotItem(X, Y);
      if NewHotArea <> HotArea then
      begin
         if (HotArea is TExplorerBarGroupItem) then
         begin
            (HotArea as TExplorerBarGroupItem).State:= [];
            (HotArea as TExplorerBarGroupItem).MouseInControl:= False;
         end;

         OldHotArea := HotArea;
         HotArea := NewHotArea;
         if (MousePosition = mhtItem) then
         begin
            if (HotArea as TExplorerBarGroupItem).Hint <> '' then
            begin
               ShowHint := True;
               Hint := (FHotArea as TExplorerBarGroupItem).Hint;
            end;
         end
         else
            ShowHint := False;

         if (OldHotArea <> nil) and (OldHotArea is TExplorerBarGroupItem) then OldHotArea := (OldHotArea as TExplorerBarGroupItem).ExplorerGroup;
         if (NewHotArea <> nil) and (NewHotArea is TExplorerBarGroupItem) then NewHotArea := (NewHotArea as TExplorerBarGroupItem).ExplorerGroup;
         if OldHotArea <> nil then
            with OldHotArea as TExplorerBarGroup do
            begin
               DrawX := FTitleRect.Left;
               DrawY := FTitleRect.Top - ICON_SIDE + ThemeElements.Height;
               Draw(FPaintBox.FBuffer, DrawX, DrawY);
            end;
         if NewHotArea <> nil then
            with NewHotArea as TExplorerBarGroup do
            begin
               DrawX := FTitleRect.Left;
               DrawY := FTitleRect.Top - ICON_SIDE + ThemeElements.Height;
               Draw(FPaintBox.FBuffer, DrawX, DrawY);
            end;
         if FHotArea = nil then
            (Sender as TFlickerFreePaintBox).Cursor := crDefault
         else
         begin
            (Sender as TFlickerFreePaintBox).Cursor := crHandPoint;
         end;
         FPaintBox.Paint;
      end;
   end;
end;

{-----------------------------------------------}

procedure TExplorerBar.PbMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
   NewHotArea: TObject;
begin
   NewHotArea := GetHotItem(X, Y);
   if (NewHotArea = HotArea) and (NewHotArea <> nil) then
   begin
//      if (NewHotArea is TExplorerBarGroupItem) then
//      begin
//         (NewHotArea as TExplorerBarGroupItem).OnItemClick(Self);
//         if Assigned(FOnClick) then fOnClick(Self, (NewHotArea as TExplorerBarGroupItem).Identifier);

      if NewHotArea is TExplorerBarGroup then
         with (NewHotArea as TExplorerBarGroup) do
         begin
            OpenClose;
         end;
   end;

   if HotArea = nil then
      (Sender as TFlickerFreePaintBox).Cursor := crDefault
   else
   begin
      (Sender as TFlickerFreePaintBox).Cursor := crHandPoint;
   end;
   Repaint;
end;

procedure TExplorerBar.PbMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
   NewHotArea: TObject;
begin
   NewHotArea := GetHotItem(X, Y);
   if (NewHotArea = HotArea) and (NewHotArea <> nil) then
      if (NewHotArea is TExplorerBarGroupItem) then
      begin
         try
            if (NewHotArea as TExplorerBarGroupItem).Enabled then
            begin
               (NewHotArea as TExplorerBarGroupItem).DoMouseDown(Sender,X,Y);
               (NewHotArea as TExplorerBarGroupItem).OnItemClick(Self);
               if Assigned(FOnClick) then fOnClick(Self, (NewHotArea as TExplorerBarGroupItem).Identifier);
            end;
         except
         end;
      end;

   if HotArea = nil then
      (Sender as TFlickerFreePaintBox).Cursor := crDefault
   else
   begin
      (Sender as TFlickerFreePaintBox).Cursor := crHandPoint;
   end;
   Repaint;
end;


{-----------------------------------------------}

procedure TExplorerBar.PaintBuffer(Sender: TObject; Buffer: TBitmap);
var
   I: Integer;
   X, Y: Integer;
begin
   if FUpdating then Exit;
   Buffer.Canvas.Brush.Color := ColourSet.Background;
   Buffer.Canvas.Font.Name:= 'Tahoma';
   Buffer.Canvas.FillRect(Buffer.Canvas.ClipRect);
   X := X_OFFSET;
   Y := Y_OFFSET;
   for I := 0 to fGroups.Count - 1 do
   begin
      if Assigned(OnBeforeDrawGroup) then OnBeforeDrawGroup(fGroups[i]);
      fGroups[I].Draw(Buffer, X, Y);
      if Assigned(OnAfterDrawGroup) then OnAfterDrawGroup(fGroups[i]);
   end;
end;

{-----------------------------------------------}

procedure TExplorerBar.SetGroupIcons(Value: TImageList);
begin
   FGroupIcons := Value;
end;

{-----------------------------------------------}

procedure TExplorerBar.Measure;
var
   I: Integer;
   X, Y: Integer;
   Closing: Boolean;
   NewPbHeight: Integer;
begin
  //resize FpaintBox & FpaintBox.Buffer
   Y := Y_OFFSET;
   X := X_OFFSET;
   Closing := False;
   for I := 0 to fGroups.Count - 1 do
      with fGroups[I] do
      begin
         Measure(FPaintBox.FBuffer, X, Y);
         if State = gsClosing then
            Closing := True;
      end;
   if Closing and (Y < ClientHeight - 1) then
      NewPbHeight := ClientHeight - 1
   else
      NewPbHeight := Y;
   if NewPbHeight <> FPaintBox.Height then
   begin
      FPaintBox.Height := NewPbHeight;
      FPaintBox.Resize;
   end;
   FGroupsHeight := Y;
   Resize;
end;

{-----------------------------------------------}

procedure TExplorerBar.SetItemIcons(Value: TImageList);
begin
   FItemIcons := Value;
end;

{-----------------------------------------------}

procedure TExplorerBar.Timer(Sender: TObject);
begin
   Inc(FAnimStep);
   if FAnimStep >= ANIM_COUNT then
   begin
      FTimer.Enabled := False;
      FPaintBox.NoDeform;
   end
   else
      FPaintBox.ApplyDeform(1, FAnimStep / ANIM_COUNT, (FAnimStep / ANIM_COUNT) * (FAnimStep / ANIM_COUNT));
   FPaintBox.Paint;
end;

{-----------------------------------------------}

constructor TExplorerBar.Create(AOwner: TComponent);
begin
   inherited Create(AOwner);
   ColourSet:= TExplorerBarColourSet.Create(Self);
   ThemeElements:= TExplorerBarThemeElements.Create(Self);
   FUpdating := False;
   FAnimate := True;
   ControlStyle := [csAcceptsControls, csCaptureMouse, csClickEvents, csSetCaption, csDoubleClicks,csActionClient];
   Width := DEFAULT_WIDTH;
   FOldWidth := DEFAULT_WIDTH;
   Height := DEFAULT_HEIGHT;
   ParentColor := False;
   VertScrollBar.Smooth := True;
   VertScrollBar.Tracking := True;
   VertScrollBar.ButtonSize := SCROLLBAR_WIDTH;
   VertScrollBar.Visible := true;
   HorzScrollBar.Visible := true;
   HorzScrollBar.Tracking:= True;
   HorzScrollBar.Smooth:= True;
   AutoScroll:= True;
   FPaintBox := TFlickerFreePaintBox.Create(Self);
   fGroups := TExplorerBarGroups.Create(Self);
   with FPaintBox do
   begin
      Parent := Self;
      SetBounds(0, 0, Self.Width, 1);
      Resize;
      OnMouseMove := PbMouseMove;
      OnMouseUp := PbMouseUp;
      OnMouseDown:= PBMouseDown;
      OnPaint := PaintBuffer;
   end;
   FGroupIcons := nil;
   FItemIcons := nil;
   FNeedResize := True;
   FHotArea := nil;
   FTimer := TTimer.Create(nil);
   FTimer.Enabled := False;
   FTimer.OnTimer := Timer;
   FTimer.Interval := ANIM_SPEED;
end;

{-----------------------------------------------}

procedure TExplorerBar.Resize;
var
   I: Integer;
   NeedRefresh: Boolean;
begin //override
   inherited;
   NeedRefresh := False;
   if (ClientHeight >= FGroupsHeight) and VertScrollBar.Visible then
   begin
      VertScrollBar.Visible := False;
      NeedRefresh := True;
   end
   else if (ClientHeight < FGroupsHeight) and not VertScrollBar.Visible then
      begin
         VertScrollBar.Visible := True;
         NeedRefresh := True;
      end;
   if Width <> FOldWidth then
   begin
      FOldWidth := Width;
      FPaintBox.Width := Width;
      FPaintBox.Resize;
      NeedRefresh := True;
   end;
   if NeedRefresh then
   begin
      for I := 0 to fGroups.Count - 1 do
      begin
         fGroups[I].FWidth := ClientWidth - 2 * X_OFFSET;
         fGroups[I].Measure;
      end;
      Measure;
      Repaint;
   end;
end;

{-----------------------------------------------}

destructor TExplorerBar.Destroy;
begin //override
   FPaintBox.Free;
   fGroups.Free;
   FTimer.Free;
   ColourSet.Free;
   ThemeElements.Free;
   inherited Destroy;
end;

procedure TExplorerBar.DoOnCustomDrawBarItem(BarItem:TObject; Bitmap: TBitmap; var X, Y, Width: Integer);
begin
   if Assigned(OnCustomDrawBarItem) then OnCustomDrawBarItem(Self,BarItem,Bitmap,X,Y,Width);
end;

procedure TExplorerBar.DoOnCustomMeasureBarItem(BarItem:TObject; Bitmap: TBitmap; var X, Y, Width: Integer);
begin
   if Assigned(OnCustomMeasureBarItem) then OnCustomMeasureBarItem(Self,BarItem,Bitmap,X,Y,Width);
end;

procedure TExplorerBar.DoOnItemClickEvent(Item: TObject; ItemIndex, GroupIndex: Integer);
begin
   if Assigned(OnItemClick) then OnItemClick(Self,Item,ItemIndex,GroupIndex);
end;

{-----------------------------------------------}


procedure TExplorerBar.BeginUpdate;
begin
   FUpdating := True;
end;

{-----------------------------------------------}

procedure TExplorerBar.EndUpdate;
begin
   FUpdating := False;
   Measure;
end;

{-----------------------------------------------}

procedure TExplorerBar.Repaint;
begin //override
   if FUpdating then Exit;
   inherited Repaint;
   if Assigned(OnBeforeDraw) then OnBeforeDraw(Self);
   if FNeedResize then
   begin
      Measure;
      FNeedResize := False;
   end;
   FPaintBox.Repaint;
   if Assigned(OnAfterDraw) then OnAfterDraw(Self);
end;

procedure TExplorerBar.ShowAnimate;
begin
   if FUpdating or not FAnimate then Exit;
   FAnimStep := 0;
   if FNeedResize then
   begin
      Measure;
      FNeedResize := False;
   end;
   FTimer.Enabled := True;
   Timer(FTimer);
end;


procedure TExplorerBar.FocusToGroup(dwIndex: Integer);
begin
   CloseAllGroups;
   BeginUpdate;
   Groups[dwIndex].OpenClose;
   EndUpdate;
end;

procedure TExplorerBar.CloseAllGroups;
var
   i:Integer;
begin
   BeginUpdate;
   for i := 0 to Groups.Count - 1 do
   begin
      if Groups[i].State = gsOpen then Groups[i].OpenClose;
   end;
   EndUpdate;
end;


{ TExplorerBarGroupItems }

function TExplorerBarGroupItems.Add(aItem:TExplorerBarGroupItem):TExplorerBarGroupItem;
var
   idx:Integer;
begin
   idx:= inherited Add(aItem);
   aItem.ExplorerGroup:= Self.Group;
   aItem.Index:= idx;
   result:= aItem;
end;


constructor TExplorerBarGroupItems.Create(fGroup: TExplorerBarGroup);
begin
   inherited Create(True);
   Group:= fGroup;
end;

destructor TExplorerBarGroupItems.Destroy;
begin
   inherited;
end;

function TExplorerBarGroupItems.GetItem(Index: Integer): TExplorerBarGroupItem;
begin
   result:= TExplorerBarGroupItem(inherited Items[Index]);
end;

function TExplorerBarGroupItems.IndexOf(aItem: TExplorerBarGroupItem): Integer;
begin
   result:= inherited IndexOf(aItem);
end;

function TExplorerBarGroupItems.Remove(aItem: TExplorerBarGroupItem): Integer;
begin
   result:= inherited Remove(aItem);
end;

procedure TExplorerBarGroupItems.SetItem(Index: Integer; Value: TExplorerBarGroupItem);
begin
   inherited Items[Index]:= Value;
end;


{ TDefaultBarItemView }

procedure TDefaultBarItemViewer.Draw(Bitmap: TBitmap; var X, Y, Width: Integer);
begin
end;

procedure TDefaultBarItemViewer.Measure(Bitmap: TBitmap; var X, Y, Width: Integer);
begin
end;

{ TCustomBarItemViewer }

constructor TCustomBarItemViewer.Create(aItem: TExplorerBarGroupItem);
begin
   inherited Create;
   Item:= aItem;
//   ShowClientArea:= True;
end;


procedure TCustomBarItemViewer.Draw(Bitmap: TBitmap; var X, Y, Width: Integer);
begin
   if ShowClientArea then
   begin
      Bitmap.Canvas.Brush.Color:= Item.ExplorerGroup.ExplorerBar.ColourSet.TextHot;
      Bitmap.Canvas.FrameRect(Item.ClientAreaRectangle);
      Bitmap.Canvas.Brush.Style:= bsClear;
   end;
   Inc(Y,Y_OFFSET div 2);
end;

procedure TCustomBarItemViewer.DrawCaption(Bitmap: TBitmap;var X, Y:Integer; var aRect:TRect; dwFlags:Integer);
var
   DrawFlags:Integer;
begin
   DrawFlags:= DT_LEFT;
   if Item.WordWrap then
      DrawFlags:= DrawFlags or DT_WORDBREAK or DT_WORD_ELLIPSIS or DT_TOP or DT_EDITCONTROL
   else
      DrawFlags:= DrawFlags or DT_SINGLELINE or DT_END_ELLIPSIS;

//      DrawText(Bitmap.Canvas.Handle,PChar(Item.Caption),Length(Item.Caption),aRect,DrawFlags);
//      DrawFlags:= DrawFlags and not DT_CALCRECT;
   DrawText(Bitmap.Canvas.Handle,PChar(Item.Caption),Length(Item.Caption),aRect,DrawFlags);
   Y:= aRect.Bottom + (Y_OFFSET div 2);
   Item.ClientAreaRectangle:= aRect;
end;

procedure TCustomBarItemViewer.Measure(Bitmap: TBitmap; var X, Y, Width: Integer);
var
   DrawFlags:Integer; ARect:TRect;
begin
   Bitmap.Canvas.Font.Style:= Item.FontStyle;
   DrawFlags:= DT_LEFT;
   if Item.WordWrap then
   begin
//        SetRect(aRect,0,0,Width,Width);
//        DrawFlags:= DrawFlags or DT_WORDBREAK or DT_WORD_ELLIPSIS or DT_TOP or DT_EDITCONTROL or DT_CALCRECT;
//        DrawText(Bitmap.Canvas.Handle,PChar(Item.Caption),Length(Item.Caption),aRect,DrawFlags);
//        Inc(Y,aRect.Bottom);
      Inc(Y,TExplorerBarText.HeightOf(Bitmap.Canvas,Item.Caption,Width) + ICON_OFFSET)
   end
   else
      Inc(Y,Bitmap.Canvas.TextHeight('Yy') + ICON_OFFSET);
   Inc(Y, Y_OFFSET div 2);
end;


procedure TCustomBarItemViewer.DrawIcon(Bitmap: TBitmap; var X, Y, Width: Integer);
begin
   if (Item.IconIndex <> UNKNOWN_VALUE) and Assigned(Item.ExplorerGroup.ExplorerBar.ItemIcons) then
   begin
      Item.ExplorerGroup.ExplorerBar.ItemIcons.Draw(Bitmap.Canvas, X + X_OFFSET, Y, Item.IconIndex);
      X:= Item.ExplorerGroup.ExplorerBar.ItemIcons.Width + (X_OFFSET * 2) + ICON_OFFSET;
   end
   else
      X:= X + (X_OFFSET);
end;


{ TExplorerBarColourSet }

constructor TExplorerBarColourSet.Create(aBar: TExplorerBar);
begin
   inherited Create;
   Bar:= aBar;
end;

procedure TExplorerBarColourSet.LoadFromFile(const lpzFilename,lpzThemeName: string);
var
   ini:TINIfile;
begin
   ini:= TIniFile.Create(lpzFilename);
   try
      if ini.SectionExists(lpzThemeName) then
      begin
         ini.ReadString(lpzThemeName,'Name',Self.Bar.ThemeElements.ThemeName);
         ini.ReadInteger(lpzThemeName, 'Text Hot',TColor(TextHot));
         ini.ReadInteger(lpzThemeName,'Text',TColor(Text));
         ini.ReadInteger(lpzThemeName,'Border',TColor(Border));
         ini.ReadInteger(lpzThemeName,'Background',TColor(Background));
         ini.ReadInteger(lpzThemeName, 'Group Background',TColor(GroupBackground));
         ini.ReadInteger(lpzThemeName, 'Special Text',TColor(SpecialText));
         ini.ReadInteger(lpzThemeName, 'Special Text Hot',TColor(SpecialTextHot));
      end;

   finally
      ini.Free;
   end;
end;

procedure TExplorerBarColourSet.SavetoFile(const lpzFilename: string);
var
   ini:TINIfile; aName:string;
begin
   aName:= Self.Bar.ThemeElements.ThemeName;
   ini:= TIniFile.Create(lpzFilename);
   try
      ini.EraseSection(Self.Bar.ThemeElements.ThemeName);
      ini.WriteString(aName,'Name',Self.Bar.ThemeElements.ThemeName);
      ini.WriteInteger(aName,'Text Hot',Integer(TextHot));
      ini.WriteInteger(aName,'Text',Integer(Text));
      ini.WriteInteger(aName,'Border',Integer(Border));
      ini.WriteInteger(aName,'Background',Integer(Background));
      ini.WriteInteger(aName,'Group Background',Integer(GroupBackground));
      ini.WriteInteger(aName,'Special Text',Integer(SpecialText));
      ini.WriteInteger(aName,'Special Text Hot',Integer(SpecialTextHot));
   finally
      ini.Free;
   end;
end;





{ TExplorerBarThemeElements }

procedure TExplorerBarThemeElements.BlendFillRect(Dest: TBitmap; ARect: TRect; Alpha: Extended);
var
   I, J: Integer;OutRect: TRect;PDest: PByteArray;R, G, B: Byte;
begin
   if Alpha >= 1 then
      Dest.Canvas.FillRect(ARect)
   else if Alpha > 0 then
      begin
         if Dest.PixelFormat <> pf24bit then
            Dest.PixelFormat := pf24bit;
         if IntersectRect(OutRect, ARect, Dest.Canvas.ClipRect) then
         begin
            R := GetRValue(Dest.Canvas.Brush.Color);
            G := GetGValue(Dest.Canvas.Brush.Color);
            B := GetBValue(Dest.Canvas.Brush.Color);
            for J := OutRect.Top to OutRect.Bottom - 1 do
            begin
               PDest := Dest.ScanLine[J];
               for I := OutRect.Left to OutRect.Right - 1 do
               begin //blending
                  PDest[I * 3] := Round(((1 - Alpha) * PDest[I * 3] + Alpha * B));
                  PDest[I * 3 + 1] := Round(((1 - Alpha) * PDest[I * 3 + 1] + Alpha * G));
                  PDest[I * 3 + 2] := Round(((1 - Alpha) * PDest[I * 3 + 2] + Alpha * R));
               end;
            end;
         end;
      end;
end;

constructor TExplorerBarThemeElements.Create(aBar: TExplorerBar);
begin
   inherited Create;
   Bar:= aBar;
   ThemeNames:= TStringList.Create;
   ThemeNames.Add('Blue');
   ThemeNames.Add('Silver');
   ThemeNames.Add('Olive');
   ThemeNames.Add('Orange');
   ThemeNames.Add('Blue Flat');

   BlueElements:= TBitmap.Create;
   OliveElements:= TBitmap.Create;
   SilverElements:= TBitmap.Create;
   OrangeElements:= TBitmap.Create;
   BlueFlatElements:= TBitmap.Create;

   BlueElements.LoadFromResourceName(hInstance, 'THEME_BLUE');
   OliveElements.LoadFromResourceName(hInstance, 'THEME_OLIVE');
   SilverElements.LoadFromResourceName(hInstance, 'THEME_SILVER');
   OrangeElements.LoadFromResourceName(hInstance, 'THEME_ORANGE');
   BlueFlatElements.LoadFromResourceName(hInstance, 'THEME_BLUE_FLAT');

   ArrowUp:= TBitmap.Create;
   ArrowUp.LoadFromResourceName(hInstance, 'ARROW_UP');
   ArrowUp.PixelFormat := pf8bit;

   ArrowDown:= TBitmap.Create;
   ArrowDown.LoadFromResourceName(hInstance, 'ARROW_DOWN');
   ArrowDown.PixelFormat := pf8bit;
   set_CheckBoxDimensions;
end;

destructor TExplorerBarThemeElements.Destroy;
begin
   ThemeNames.Free;
   BlueElements.Free;
   OliveElements.Free;
   SilverElements.Free;
   OrangeElements.Free;
   BlueFlatElements.Free;
   ArrowDown.Free;
   ArrowUp.Free;
   inherited;
end;

procedure TExplorerBarThemeElements.DrawHeader(Bitmap: TBitmap; X, Y, Width: Integer; SpecialGroup, Closed: Boolean;TextColor: TColor);
begin
   with Bitmap.Canvas do
   begin
      Brush.Color := IfThen(SpecialGroup, clActiveCaption, clBtnFace);
      FillRect(Rect(X, Y + ICON_SIDE - Height, X + Width, Y + ICON_SIDE));
      Brush.Color := TextColor;
      if Closed then
         DrawMask(Bitmap, Width - X_OFFSET, Y + 2 * Y_OFFSET, ArrowDown)
      else
         DrawMask(Bitmap, Width - X_OFFSET, Y + 2 * Y_OFFSET, ArrowUp);
   end;
end;

procedure TExplorerBarThemeElements.DrawMask(Dest: TBitmap; X, Y: Integer; Src: TBitmap);
var
   I, J: Integer; X2, Y2: Integer;PSrc:PByteArray;
begin
   if (Y >= Dest.Height) or (X >= Dest.Width) then Exit;
   Y2 := Min(Src.Height, Dest.Height - Y) - 1;
   X2 := Min(Src.Width, Dest.Width - X) - 1;
   for J := 0 to Y2 do
   begin
      PSrc := Src.ScanLine[J];
      for I := 0 to X2 do
         if PSrc[I] = 0 then
         begin
            Dest.Canvas.Pixels[X + I, Y + J] := Dest.Canvas.Brush.Color;
         end;
   end;
end;

procedure TExplorerBarThemeElements.DrawThemedHeader(Bitmap, Skin: TBitmap; X, Y, Width: Integer; SpecialGroup, Closed,MouseOver: Boolean);
begin
   SetStretchBltMode(Bitmap.Canvas.Handle, COLORONCOLOR);
   if SpecialGroup then
   begin
      BitBlt(Bitmap.Canvas.Handle, X, Y + ICON_SIDE - Height, SKIN_LEFT_PART_WIDTH, Height, Skin.Canvas.Handle, ElementRect(terSpecialTitleLeft).Left, ElementRect(terSpecialTitleLeft).Top, SRCCOPY);
      StretchBlt(Bitmap.Canvas.Handle, X + SKIN_LEFT_PART_WIDTH, Y + ICON_SIDE - Height, Width - SKIN_LEFT_PART_WIDTH - SKIN_RIGHT_PART_WIDTH,
         Height, Skin.Canvas.Handle, ElementRect(terSpecialTitleCenter).Left, ElementRect(terSpecialTitleCenter).Top,
         ElementRect(terSpecialTitleCenter).Right - ElementRect(terSpecialTitleCenter).Left, ElementRect(terSpecialTitleCenter).Bottom - ElementRect(terSpecialTitleCenter).Top, SRCCOPY);
      BitBlt(Bitmap.Canvas.Handle, X + Width - SKIN_RIGHT_PART_WIDTH, Y + ICON_SIDE - Height, SKIN_RIGHT_PART_WIDTH, Height, Skin.Canvas.Handle,
         IfThen(Closed, IfThen(MouseOver, ElementRect(terSpecialTitleRightDownOn).Left, ElementRect(terSpecialTitleRightDownOff).Left), IfThen(MouseOver, ElementRect(terSpecialTitleRightUpOn).Left, ElementRect(terSpecialTitleRightUpOff).Left)),
         IfThen(Closed, IfThen(MouseOver, ElementRect(terSpecialTitleRightDownOn).Top, ElementRect(terSpecialTitleRightDownOff).Top), IfThen(MouseOver, ElementRect(terSpecialTitleRightUpOn).Top, ElementRect(terSpecialTitleRightUpOff).Top)), SRCCOPY);
   end
   else
   begin
      BitBlt(Bitmap.Canvas.Handle, X, Y + ICON_SIDE - Height, SKIN_LEFT_PART_WIDTH, Height, Skin.Canvas.Handle, ElementRect(terTitleLeft).Left, ElementRect(terTitleLeft).Top, SRCCOPY);
      StretchBlt(Bitmap.Canvas.Handle, X + SKIN_LEFT_PART_WIDTH, Y + ICON_SIDE - Height, Width - SKIN_LEFT_PART_WIDTH - SKIN_RIGHT_PART_WIDTH,
         Height, Skin.Canvas.Handle, ElementRect(terTitleCenter).Left, ElementRect(terTitleCenter).Top,
         ElementRect(terTitleCenter).Right - ElementRect(terTitleCenter).Left, ElementRect(terTitleCenter).Bottom - ElementRect(terTitleCenter).Top, SRCCOPY);
      BitBlt(Bitmap.Canvas.Handle, X + Width - SKIN_RIGHT_PART_WIDTH, Y + ICON_SIDE - Height, SKIN_RIGHT_PART_WIDTH, Height, Skin.Canvas.Handle,
         IfThen(Closed, IfThen(MouseOver, ElementRect(terTitleRightDownOn).Left, ElementRect(terTitleRightDownOff).Left), IfThen(MouseOver, ElementRect(terTitleRightUpOn).Left, ElementRect(terTitleRightUpOff).Left)),
         IfThen(Closed, IfThen(MouseOver, ElementRect(terTitleRightDownOn).Top, ElementRect(terTitleRightDownOff).Top), IfThen(MouseOver, ElementRect(terTitleRightUpOn).Top, ElementRect(terTitleRightUpOff).Top)), SRCCOPY);
   end;
end;

function TExplorerBarThemeElements.ElementRect(dwType: TThemeElementRectType): TRect;
begin
   case dwtype of
      terTitleLeft                :result:= Rect(7,1,12,24);
      terTitleCenter              :result:= Rect(13,1,163,24);
      terTitleRightUpOff          :result:= Rect(164,1,189,24);
      terTitleRightUpOn           :result:= Rect(190,1,215,24);
      terTitleRightDownOff        :result:= Rect(216,1,241,24);
      terTitleRightDownOn         :result:= Rect( 242,1,267,24);
      terSpecialTitleLeft         :result:= Rect(7,25,12,48);
      terSpecialTitleCenter       :result:= Rect(13,25,163,48);
      terSpecialTitleRightUpOff   :result:= Rect(164,25,189,48);
      terSpecialTitleRightUpOn    :result:= Rect(190,25,215,48);
      terSpecialTitleRightDownOff :result:= Rect(216,25,241,48);
      terSpecialTitleRightDownOn  :result:= Rect(242,25,267,48);
   end;
end;

function TExplorerBarThemeElements.get_Height: Integer;
begin
   result:= 23;
end;

function TExplorerBarThemeElements.get_Theme: TBitmap;
begin
   if FTheme = 'Blue' then result:= BlueElements;
   if FTheme = 'Silver' then result:= SilverElements;
   if FTheme = 'Olive' then result:= OliveElements;
   if FTheme = 'Orange' then result:= OrangeElements;
   if FTheme = 'Blue Flat' then result:= BlueFlatElements;
end;

procedure TExplorerBarThemeElements.SetThemeName(const Value: String);
begin
   fTheme := Value;
   with Bar do
   begin
      ColourSet.Background:= Theme.Canvas.Pixels[1, 1];
      ColourSet.GroupBackground:= Theme.Canvas.Pixels[1, 7];
      ColourSet.Border:= Theme.Canvas.Pixels[1, 13];
      ColourSet.Text:= Theme.Canvas.Pixels[1, 19];
      ColourSet.TextHot:= Theme.Canvas.Pixels[1, 25];
      ColourSet.SpecialText:= Theme.Canvas.Pixels[1, 31];
      ColourSet.SpecialTextHot:= Theme.Canvas.Pixels[1, 37];
      Color := ColourSet.Background;
      Repaint;
   end;
//      ColourSet.Background:= clWhite; //clWindow;
//      ColourSet.GroupBackground:= clWindow;
//      ColourSet.Border:= clBtnFace;
//      ColourSet.Text:= clBtnText;
//      ColourSet.TextHot:= clBtnText;
//      ColourSet.SpecialText:= clCaptionText;
//      ColourSet.SpecialTextHot:= clCaptionText;
//      Color := clWhite; //clWindow;
end;

procedure TExplorerBarThemeElements.set_CheckBoxDimensions;
begin
   with TBitmap.Create do
      try
         Handle := LoadBitmap(0, PChar(32759));
         CheckBoxWidth := Width div 4;
         CheckBoxHeight := Height div 3;
      finally
         Free;
      end;
end;

end.
