unit codecraft.explorerbar.utils;

interface
uses Windows,Classes,Graphics;

type
   TExplorerBarRenderer = class(TObject)
   public
      class procedure DrawAlpha(Dest: TBitmap; BgColor: TColor; X, Y: Integer; SrcRect: TRect; Src: TBitmap; Alpha: Extended = 1);
   end;

   TExplorerBarText = class(TObject)
   public
      class function Parse(Canvas: TCanvas; var Line: string; MaxWidth: Integer): string;
      class function ClipLine(Canvas: TCanvas; Line: string; MaxWidth: Integer): string;
      class function HeightOf(Canvas: TCanvas; const Line: string; MaxWidth: Integer):Integer;
   end;


implementation
uses
   SysUtils,Math;

class procedure TExplorerBarRenderer.DrawAlpha(Dest: TBitmap; BgColor: TColor; X, Y: Integer; SrcRect: TRect; Src: TBitmap; Alpha: Extended = 1);
var
   I, J: Integer;
   RTmp, OutRect: TRect;
   PSrc, PDest: PByteArray;
   R, G, B: Byte;
begin
   if Alpha >= 1 then //if no alpha blending
      Dest.Canvas.CopyRect(Rect(X, Y, X + SrcRect.Right - SrcRect.Left, Y + SrcRect.Bottom - SrcRect.Top), Src.Canvas, SrcRect)
   else
   begin
      if Src.PixelFormat <> pf24bit then
         Src.PixelFormat := pf24bit;
      if Dest.PixelFormat <> pf24bit then
         Dest.PixelFormat := pf24bit;
      RTmp := Rect(Max(0, X), Max(0, Y), Min(Dest.Width - 1, X + SrcRect.Right - SrcRect.Left - 1), Min(Dest.Height - 1, Y + SrcRect.Bottom - SrcRect.Top - 1));
      if IntersectRect(OutRect, RTmp, Dest.Canvas.ClipRect) then
      begin
         R := GetRValue(BgColor);
         G := GetGValue(BgColor);
         B := GetBValue(BgColor);
         for J := OutRect.Top to OutRect.Bottom do
         begin
            PSrc := Src.ScanLine[J - Y + SrcRect.Top];
            PDest := Dest.ScanLine[J];
            for I := OutRect.Left to OutRect.Right do
            begin //blending
               PDest[I * 3] := Round(((1 - Alpha) * B + Alpha * PSrc[(I - X + SrcRect.Left) * 3]));
               PDest[I * 3 + 1] := Round(((1 - Alpha) * G + Alpha * PSrc[(I - X + SrcRect.Left) * 3 + 1]));
               PDest[I * 3 + 2] := Round(((1 - Alpha) * R + Alpha * PSrc[(I - X + SrcRect.Left) * 3 + 2]));
            end;
         end;
      end;
   end;
end;

{-----------------------------------------------
parse a line to fit in a specified width}

class function TExplorerBarText.HeightOf(Canvas: TCanvas; const Line: string; MaxWidth: Integer): Integer;
var
   dwOffset: Integer; r:TRect;
begin
   Result := 0;
   SetRect(r,0,0,MaxWidth,MaxWidth);
   dwOffset:= DrawText(Canvas.Handle,PChar(Line),Length(Line),r,DT_CALCRECT or DT_WORDBREAK or DT_EDITCONTROL or DT_TOP or DT_WORD_ELLIPSIS or DT_MODIFYSTRING);
   result:= r.Bottom;// + Canvas.TextHeight('A');
end;

class function TExplorerBarText.Parse(Canvas: TCanvas; var Line: string; MaxWidth: Integer): string;
var
   I: Integer;
   Word: string;
   Stop: Boolean;
begin
   Result := '';
   Stop := False;
   while not Stop and (Line <> '') do
   begin
      I := 1;
      while (I <= Length(Line)) and (Line[I] <> ' ') do
         Inc(I);
      Word := Copy(Line, 1, I);
      if (Canvas.TextWidth(Result + Word) < MaxWidth) then
      begin
         Line := Copy(Line, I + 1, Length(Line));
         Result := Result + Word;
      end
      else
      begin
         if Result = '' then
         begin
            Line := Copy(Line, I + 1, Length(Line));
            repeat
               Delete(Word, Length(Word), 1);
            until (Canvas.TextWidth(Word + '...') < MaxWidth) or (Length(Word) < 2);
            Result := Word + '...';
         end;
         Stop := True;
      end;
   end;
end;


class function TExplorerBarText.ClipLine(Canvas: TCanvas; Line: string; MaxWidth: Integer): string;
begin
   Result := Line;
   if Canvas.TextWidth(Line) > MaxWidth then
   begin
      repeat
         System.Delete(Result, Length(Result), 1);
      until (Canvas.TextWidth(Result + '...') <= MaxWidth) or (Length(Result) < 2);
      Result := Result + '...';
   end;
end;


end.
